//****************************************************************************//
//                                                                            //
//   pyrecola.c                                                               //
//   is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2       //
//                                                                            //
//   Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and             //
//                           Sandro Uccirati                                  //
//                                                                            //
//   RECOLA2 is licenced under the GNU GPL version 3,                         //
//   see COPYING for details.                                                 //
//                                                                            //
//****************************************************************************//

#ifdef __cplusplus
extern "C" {
#endif 
#include "Python.h"
#include <recola.h>
#include <stdio.h>

static char pyrecola_doc[] = 
"This is the python interface to Recola2.";

/**********************
*  helper functions  *
**********************/

int pyobj_to_int(PyObject * obj_py)
{
#ifdef PYTHON3
  int ret = PyLong_AsLong(obj_py)
  Py_XDECREF(obj_py);
  return ret;
#else
  int ret = PyInt_AsLong(obj_py);
  Py_XDECREF(obj_py);
  return ret;
#endif
}
PyObject * int_to_pyobj(int in)
{
#ifdef PYTHON3
    return PyLong_FromLong(in);
#else
    return PyInt_FromLong(in);
#endif
}

int pyobj_pyint_check(PyObject * obj_py)
{
#ifdef PYTHON3
  return PyLong_Check(obj_py);
#else
  return PyInt_Check(obj_py);
#endif
}
/*static void build_string(PyObject *obj, const char **ret) {*/
char *pyobj_to_string(PyObject *obj) {
#ifdef PYTHON3
  PyObject* str = PyUnicode_AsUTF8String(obj);
  char *ret =  PyBytes_AS_STRING(str);
  Py_XDECREF(str);
  return ret
#else
  return PyString_AsString(obj);
#endif
}

static PyObject *
build_c_momenta(const int process_id, PyObject* momenta_py,
                double momenta_c[][4])
{
  int legs;
  get_legs_rcl(process_id, &legs);
  int numLines;
  numLines = PyList_Size(momenta_py);

  if(legs != numLines){
    return PyErr_Format(PyExc_Exception,
        "Number of particles and length of momenta do not match.\n"\
        "Given length: %d, expected length: %d",
        numLines, legs);
  }

  int i;
  for (i=0; i<numLines; i++)
  {
    PyObject *momentum_py = PyList_GetItem(momenta_py, i);
    PyObject *iter = PyObject_GetIter(momentum_py);
    if (!iter)
    {
      // error, expected list/iterator
      return PyErr_Format(PyExc_Exception,
      "Given momenta %d is not of list/iterator type.", i);
    }
    int j = 0;
    while (1) {
      PyObject *next = PyIter_Next(iter);
      if (!next) {
        // nothing left in the iterator
        break;
      }
      if (j > 3)
      {
        // error, expected four-momentum
        return PyErr_Format(PyExc_Exception,
        "Momentum %d exeeds the expected length of 4.", i);
      }
      if (!PyFloat_Check(next)) 
      {
        // error, we were expecting a complex floating point 
        return PyErr_Format(PyExc_Exception,
        "Momentum %d component %d is not a (floating point) complex.", i, j);
      }
      momenta_c[i][j] = PyFloat_AsDouble(next);
      Py_DECREF(next);
      j = j + 1;
    }
    Py_DECREF(iter);
  }
  return momenta_py;
}
static PyObject *
build_c_integer_array(const int length, PyObject* array_py,
                      int array_c[], const char *array_name)
{
  int lengthp = PyList_Size(array_py);

  if(lengthp != length){
    return PyErr_Format(PyExc_Exception,
    "Size of %s does not match the expected length.\n"\
    "Given length: %d, expected (from model/process) length: %d",
    array_name, lengthp, length);
  }
  int i;
  for (i=0; i<length; i++)
  {
    PyObject *tmp = PyList_GetItem(array_py, i);
    array_c[i] = pyobj_to_int(tmp);
  }
  return array_py;
}

static PyObject *
build_c_polvec(PyObject *polvec_py, dcomplex polvec_c[4])
{
  PyObject *iter = PyObject_GetIter(polvec_py);
  if (!iter)
  {
    // error not iterator
    return PyErr_Format(PyExc_Exception,
    "Polvec is not an iterator/list.");
  }
  int j = 0;
  while (1)
  {
    PyObject *next = PyIter_Next(iter);
    if (!next) 
    {
      // nothing left in the iterator
      break;
    }
    if (j > 3) 
    {
      // error, size exceeded
      return PyErr_Format(PyExc_Exception,
      "Polvec exeeds the expected length of 4.");
    }
    if (!PyComplex_Check(next)) {
      // error, we were expecting a complex floating point value
      return PyErr_Format(PyExc_Exception,
      "Polvec component %d is not a (floating point) complex.", j);
    }
    polvec_c[j].dr = PyComplex_RealAsDouble(next);
    polvec_c[j].di = PyComplex_ImagAsDouble(next);
    Py_DECREF(next);
    j = j + 1;
  }
  Py_DECREF(iter);
  return polvec_py;
}

/**********************
*  module input_rcl  *
**********************/

PyDoc_STRVAR(set_qcd_rescaling_doc,
"set_qcd_rescaling_rcl(val)\n\n\
Set the use of rescaling/running methods of RECOL1.\n\
Note that this only makes sense for SM-QCD-like models as\n\
the hard-coded beta functions are used.\n\
val = True  -> Enabled\n\
val = False -> Disabled");
static PyObject *
set_qcd_rescaling_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_qcd_rescaling_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_fermionloop_optimization_doc,
"set_fermionloop_optimization_rcl(val)\n\n\
Set the fermion loop optimization. Disabled by default.\n\
val = True  -> identifies equal fermion loops for degenerate fermion masses\n\
val = False -> no optimization performed");
static PyObject *
set_fermionloop_optimization_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_fermionloop_optimization_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_helicity_optimization_doc,
"set_helicity_optimization_rcl(val)\n\n\
Set the helicity optimization level to 0, 1 or 2.\n\
val = True  -> standard helicity optimization\n\
val = False -> no helicity optimization");
static PyObject *
set_helicity_optimization_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_helicity_optimization_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_colour_optimization_doc,
"set_colour_optimization_rcl(val)\n\n\
Set the colour optimization level to 0, 1 or 2.\n\
val = 0 -> no colour optimization\n\
val = 1 -> standard colour-flow optimization\n\
val = 2 -> improved colour-flow optimization\n\
           (no explicit computation of external U1 Gluons)");
static PyObject *
set_colour_optimization_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_colour_optimization_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_compute_A12_doc,
"set_compute_A12_rcl(val)\n\n\
Set whether to compute the squared loop contribution A1^2.\n\
val = True  -> Enabled\n\
val = False -> Disabled");
static PyObject *
set_compute_A12_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_compute_A12_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_masscut_doc,
"set_masscut_rcl(m)\n\n\
Sets mass limit `m` below which particles are considered massless.\n\
m = float -> positve number");
static PyObject *
set_masscut_py(PyObject *self, PyObject *args)
{
  double value;

  if (!PyArg_ParseTuple(args, "d", &value))
      return NULL;

  set_masscut_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_particle_doc,
"set_light_particle_rcl(p)\n\n\
Tag a particle `p` as a light one, omitting mass terms whenever possible.\n\
This corresponds to mass regularization.\n\
p = string -> particle name (anti-particle automatically set to light)");
static PyObject *
set_light_particle_py(PyObject *self, PyObject *args)
{
  const char* p;

  if (!PyArg_ParseTuple(args, "s", &p))
      return NULL;

  set_light_particle_rcl(p);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(unset_light_particle_doc,
"unset_light_particle_rcl(p)\n\n\
Untag a particle `p` as a light one, keeping the full mass dependence\n\
p = string -> particle name (anti-particle automatically unset to light)");
static PyObject *
unset_light_particle_py(PyObject *self, PyObject *args)
{
  const char* p;

  if (!PyArg_ParseTuple(args, "s", &p))
      return NULL;

  unset_light_particle_rcl(p);

  Py_INCREF(Py_None);
  return Py_None; 
}
//deprecated
PyDoc_STRVAR(use_dim_reg_soft_doc,
"use_dim_reg_soft_rcl\n\n\
Deprecated. Call has no effect.");
static PyObject *
use_dim_reg_soft_py(PyObject *self, PyObject *args)
{
  use_dim_reg_soft_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
//deprecated
PyDoc_STRVAR(use_mass_reg_soft_doc,
"use_mass_reg_soft_rcl(m)\n\n\
Deprecated. Call has no effect.\n\
m = float -> positive number");
static PyObject *
use_mass_reg_soft_py(PyObject *self, PyObject *args)
{
  double m;

  if (!PyArg_ParseTuple(args, "d", &m))
    return NULL;

  use_mass_reg_soft_rcl(m);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(use_recola_base_doc,
"use_recola_base_rcl(rb)\n\n\
Set whether current expressions are computed by recola or by the model file.\n\
rb = True -> compute inside recola\n\
rb = False -> compute inside model file");
static PyObject *
use_recola_base_py(PyObject *self, PyObject *args)
{
  double rb;

  if (!PyArg_ParseTuple(args, "i", &rb))
    return NULL;

  use_recola_base_rcl(rb);
  Py_INCREF(Py_None);
  return Py_None;
}
//deprecated
PyDoc_STRVAR(set_mass_reg_soft_doc,
"set_mass_reg_soft_rcl(m)\n\n\
Deprecated. Call has no effect.\n\
m = float -> positive number");
static PyObject *
set_mass_reg_soft_py(PyObject *self, PyObject *args)
{
  double m;

  if (!PyArg_ParseTuple(args, "d", &m))
    return NULL;

  set_mass_reg_soft_rcl(m);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_delta_uv_doc,
"set_delta_uv_rcl(d)\n\n\
This subroutine sets the finite part of the UV subtracted term:\n\
deltaUV = 1/ep - EulerConstant + ln(4*pi) to `d`.\n\
d = float -> deltaUV");
static PyObject *
set_delta_uv_py(PyObject *self, PyObject *args)
{
  double d;

  if (!PyArg_ParseTuple(args, "d", &d))
      return NULL;

  set_delta_uv_rcl(d);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_delta_uv_doc,
"get_delta_uv_rcl\n\n\
Gets the finite part of the UV subtracted term deltaUV.\n\
return = float -> deltaUV");
static PyObject *
get_delta_uv_py(PyObject *self, PyObject *args)
{
  double value;
  get_delta_uv_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(set_delta_ir_doc,
"set_delta_ir_rcl(d1, d2)\n\n\
This subroutine sets the finite part of the IR subtracted term:\n\
deltaIR{i} = 1/ep{^i} - EulerConstant + ln(4*pi) to `d`.\n\
d1 = float -> deltaIR{1}\n\
d2 = float -> deltaIR{2}");
static PyObject *
set_delta_ir_py(PyObject *self, PyObject *args)
{
  double d1,d2;

  if (!PyArg_ParseTuple(args, "dd", &d1, &d2))
      return NULL;

  set_delta_ir_rcl(d1, d2);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_delta_ir_doc,
"get_delta_ir_rcl\n\n\
Gets the finite parts of the IR subtracted terms deltair,deltair2.\n\
return = float, float -> (deltair, deltair2)");
static PyObject *
get_delta_ir_py(PyObject *self, PyObject *args)
{
  double d1,d2;
  get_delta_ir_rcl(&d1, &d2);
  return Py_BuildValue("dd", d1,d2);
}
PyDoc_STRVAR(set_mu_uv_doc,
"set_mu_uv_rcl(m)\n\n\
Sets the UV scale `muUV` to `m`.\n\
m = float -> muUV scale");
static PyObject *
set_mu_uv_py(PyObject *self, PyObject *args)
{
  double mu_uv;

  if (!PyArg_ParseTuple(args, "d", &mu_uv))
      return NULL;

  set_mu_uv_rcl(mu_uv);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_mu_uv_doc,
"get_mu_uv_rcl\n\n\
Returns the UV scale muUV.\n\
return = float -> muUV scale");
static PyObject *
get_mu_uv_py(PyObject *self, PyObject *args)
{
  double value;
  get_mu_uv_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(set_mu_ir_doc,
"set_mu_ir_rcl(m)\n\n\
Sets the IR scale `muIR` to `m`.\n\
m = float -> muIR scale");
static PyObject *
set_mu_ir_py(PyObject *self, PyObject *args)
{
  double mu_ir;

  if (!PyArg_ParseTuple(args, "d", &mu_ir))
      return NULL;

  set_mu_ir_rcl(mu_ir);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_mu_ir_doc,
"get_mu_ir_rcl\n\n\
Returns the IR scale muIR.\n\
return = float -> muIR scale");
static PyObject *
get_mu_ir_py(PyObject *self, PyObject *args)
{
  double value;
  get_mu_ir_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(set_mu_ms_doc,
"set_mu_ms_rcl(m)\n\n\
Sets the MS scale `muMS` to `m`.\n\
m = float -> muMS scale");
static PyObject *
set_mu_ms_py(PyObject *self, PyObject *args)
{
  double mu_ms;

  if (!PyArg_ParseTuple(args, "d", &mu_ms))
      return NULL;

  set_mu_ms_rcl(mu_ms);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_mu_ms_doc,
"get_mu_ms_rcl()\n\n\
Returns the MS scale muMS.\n\
return = float -> muMS scale");
static PyObject *
get_mu_ms_py(PyObject *self, PyObject *args)
{
  double value;
  get_mu_ms_rcl(&value);
  return Py_BuildValue("d", value);
}
//deprecated
PyDoc_STRVAR(get_renormalization_scale_doc,
"get_renormalization_scale_rcl()\n\n\
Deprecated. Calls get_mu_ms_rcl instead.");
static PyObject *
get_renormalization_scale_py(PyObject *self, PyObject *args)
{
  double value;
  get_renormalization_scale_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(get_flavour_scheme_doc,
"get_flavour_scheme_rcl()\n\n\
Returns the value of the flavour-scheme `Nf`.");
static PyObject *
get_flavour_scheme_py(PyObject *self, PyObject *args)
{
  int value;
  get_flavour_scheme_rcl(&value);
  return Py_BuildValue("i", value);
}
PyDoc_STRVAR(set_complex_mass_scheme_doc,
"set_complex_mass_scheme_rcl\n\n\
Selects the Complex-Mass scheme for the renormalization\n\
of particles.");
static PyObject *
set_complex_mass_scheme_py(PyObject *self, PyObject *args)
{
  set_complex_mass_scheme_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_on_shell_scheme_doc,
"set_on_shell_scheme_rcl\n\n\
Selects the on-shell scheme for the renormalization\n\
of particles.");
static PyObject *
set_on_shell_scheme_py(PyObject *self, PyObject *args)
{
  set_on_shell_scheme_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_momenta_correction_doc,
"set_momenta_correction_rcl(mc)\n\n\
Enable or disable the internal momentum corrections.\n\
mc = True  -> recola tries to correct the momenta\n\
mc = False -> recola does not try to correct the momenta");
static PyObject *
set_momenta_correction_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_momenta_correction_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_print_level_amplitude_doc,
"set_print_level_amplitude_rcl(n)\n\n\
Set the print level for amplitudes to `n`.\n\
n = 0 -> nothing is printed\n\
n = 1 -> The non vanishining amplitudes `A(c,h)` are printed at LO\n\
         and NLO (if NLO is computed) for each coulour structure\n\
         `c` and helicity `h`\n\
n = 2 -> In addition to `n=1`, the NLO (if computed) contributions\n\
         to `A(c,h)` from bare-loop, counterterms and rational part\n\
         are also printed");
static PyObject *
set_print_level_amplitude_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_level_amplitude_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_print_level_squared_amplitude_doc,
"set_print_level_squared_amplitude_rcl(n)\n\n\
Set the print level for squared amplitudes to `n`.\n\
n = 0 -> nothing is printed\n\
n = 1 -> The non vanishining squared amplitudes A2\n\
         summed/averaged over colour and helicities are\n\
         printed at LO and NLO (if NLO is computed)\n\
n = 2 -> In addition to n=1, the NLO (if computed) contributions\n\
         to A2 from bare-loop, counterterms and rational part\n\
         are also printed.\n\
n = 3 -> In addition to `n=2`, also the squared amplitudes A2h\n\
         summed/averaged over colour are printed at LO and NLO\n\
         (if NLO is computed) for each helicity `h`");
static PyObject *
set_print_level_squared_amplitude_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_level_squared_amplitude_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_print_level_correlations_doc,
"set_print_level_correlations_rcl(n)\n\n\
Set the print level for (colour/spin) correlations to `n`.\n\
n = 0 -> nothing is printed\n\
n = 1 -> The non vanishining colour-correlated Born squared\n\
         amplitudes A2c(n,m) are printed for each emitter `n` and\n\
         spectator `m`");
static PyObject *
set_print_level_correlations_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_level_correlations_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_print_level_parameters_doc,
"set_print_level_parameters_rcl(n)\n\n\
Set the level for printing parameters of the theory to `n`.\n\
n = 0 -> nothing is printed\n\
n = 1 -> tree-level input parameters are printed\n\
n = 2 -> tree-level input + derived parameters are printed\n\
n = 3 -> input + derived + counterterm parameters are printed\n\
n = 4 -> only counterterm parameters are printed");
static PyObject *
set_print_level_parameters_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_level_parameters_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_draw_level_branches_doc,
"set_draw_level_branches_rcl(n)\n\n\
Set the draw level of the off-shell currents to `n`.\n\
n = 0 -> the .tex file is not generated\n\
n = 1 -> the .tex file is generated without colour structures\n\
n = 2 -> the .tex file is generated with colour structures");
static PyObject *
set_draw_level_branches_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_draw_level_branches_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_print_level_RAM_doc,
"set_print_level_RAM_rcl(n)\n\n\
Not yet supported.");
static PyObject *
set_print_level_RAM_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_level_RAM_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(scale_coupling3_doc,
"scale_coupling3_rcl(fac,p1,p2,p3)\n\n\
Not yet supported.");
static PyObject *
scale_coupling3_py(PyObject *self, PyObject *args)
{
  PyObject *value;
  const char *p1,*p2,*p3;
  if (!PyArg_ParseTuple(args, "Osss", value,&p1,&p2,&p3))
      return NULL;

  if (!PyComplex_Check(value)) {
    /* TODO:  <25-07-17, Jean-Nicolas Lang> */
  }

  dcomplex fac;
  fac.dr = PyComplex_RealAsDouble(value);
  fac.di = PyComplex_ImagAsDouble(value);
  
  scale_coupling3_rcl(fac,p1,p2,p3);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(scale_coupling4_doc,
"scale_coupling4_rcl(fac,p1,p2,p3,p4)\n\n\
Not yet supported.");
static PyObject *
scale_coupling4_py(PyObject *self, PyObject *args)
{
  PyObject *value;
  const char *p1,*p2,*p3,*p4;
  if (!PyArg_ParseTuple(args, "Dssss", value,&p1,&p2,&p3,&p4))
      return NULL;

  dcomplex fac;
  fac.dr = PyComplex_RealAsDouble(value);
  fac.di = PyComplex_ImagAsDouble(value);
  
  scale_coupling4_rcl(fac,p1,p2,p3,p4);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(switchoff_coupling3_doc,
"switchoff_coupling3_rcl(p1,p2,p3)\n\n\
Not yet supported.");
static PyObject *
switchoff_coupling3_py(PyObject *self, PyObject *args)
{
  const char *p1,*p2,*p3;
  if (!PyArg_ParseTuple(args, "sss",&p1,&p2,&p3))
      return NULL;

  switchoff_coupling3_rcl(p1,p2,p3);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(switchoff_coupling4_doc,
"switchoff_coupling3_rcl(p1,p2,p3,p4)\n\n\
Not yet supported.");
static PyObject *
switchoff_coupling4_py(PyObject *self, PyObject *args)
{
  const char *p1,*p2,*p3,*p4;
  if (!PyArg_ParseTuple(args,"ssss",&p1,&p2,&p3,&p4))
      return NULL;

  switchoff_coupling4_rcl(p1,p2,p3,p4);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(switchon_resonant_selfenergies_doc,
"switchon_resonant_selfenergies_rcl\n\n\
Sets the value of `resSE` to .true.");
static PyObject *
switchon_resonant_selfenergies_py(PyObject *self, PyObject *args)
{
  switchon_resonant_selfenergies_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(switchoff_resonant_selfenergies_doc,
"switchon_resonant_selfenergies_rcl\n\n\
Sets the value of `resSE` to .true.");
static PyObject *
switchoff_resonant_selfenergies_py(PyObject *self, PyObject *args)
{
  switchoff_resonant_selfenergies_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_dynamic_settings_doc,
"set_dynamic_settings_rcl(i)\n\n\
Sets the dynamic_settings flag to `i`.\n\
Not yet supported.");
static PyObject *
set_dynamic_settings_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_dynamic_settings_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_ifail_doc,
"set_ifail_rcl(i)\n\n\
Sets the ifail flag to `i`.\n\
The ifail flag regulates the behaviour of Recola in case of an error.\n\
ifail = -1 -> the code does not stop when an error occurs\n\
ifail =  0 -> the code stops, when an error occurs (DEFAULT)");
static PyObject *
set_ifail_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_ifail_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_ifail_doc,
"get_ifail_rcl()\n\n\
Returns the value of the ifail flag in.\n\
The ifail flag regulates the behaviour of Recola in case of an error.\n\
return = 0 -> no errors occured\n\
return = 1 -> an error occured because of a wrong call caused by the user\n\
return = 2 -> an error occured because of an internal bug of Recola");
static PyObject *
get_ifail_py(PyObject *self, PyObject *args)
{
  int value;
  get_ifail_rcl(&value);
  return Py_BuildValue("i", value);
}
PyDoc_STRVAR(set_iwarn_doc,
"set_iwarn_rcl(i)\n\n\
Sets the iwarn flag to `i`.\n\
The iwarn flag regulates the behaviour of Recola in case of an error.\n\
iwarn = -1 -> the code does not stop when a warning occurs (DEFAULT)\n\
iwarn =  0 -> the code stops, when a warning occurs");
static PyObject *
set_iwarn_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_iwarn_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_iwarn_doc,
"get_iwarn_rcl()\n\n\
Returns the value of the iwarn flag in.\n\
The iwarn flag regulates the behaviour of Recola in case of an error.\n\
return = 0 -> no warning occured\n\
return = 1 -> a warning occured because of a wrong call caused by the user");
static PyObject *
get_iwarn_py(PyObject *self, PyObject *args)
{
  int value;
  get_iwarn_rcl(&value);
  return Py_BuildValue("i", value);
}
PyDoc_STRVAR(set_collier_output_dir_doc,
"set_collier_output_dir_rcl(dir)\n\n\
Sets the collier output folder path to 'dir' which\n\
can be a relative or absolute path.\n\
dir = string -> output folder path");
static PyObject *
set_collier_output_dir_py(PyObject *self, PyObject *args)
{
  const char* p;

  if (!PyArg_ParseTuple(args, "s", &p))
      return NULL;

  set_collier_output_dir_rcl(p);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_output_file_doc,
"set_output_file_rcl(x)\n\n\
Sets the output file name to 'x' of type string.\n\
x = string -> output file name");
static PyObject *
set_output_file_py(PyObject *self, PyObject *args)
{
  const char* p;

  if (!PyArg_ParseTuple(args, "s", &p))
      return NULL;

  set_output_file_rcl(p);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_log_mem_doc,
"set_log_mem_rcl(logmem)\n\n\
Sets the switch for writing a memorylog.\n\
logmem = False -> memory is not logged is printed.\n\
logmem = True  -> memory is logged and written to memory_rcl.log\n\
                  in the current working directory.");
static PyObject *
set_log_mem_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_log_mem_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_init_collier_doc,
"set_init_collier_rcl(initcollier)\n\n\
Sets the switch for initializing collier.\n\
initcollier = False -> collier is not initialized\n\
initcollier = True  -> collier is initialized");
static PyObject *
set_init_collier_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_init_collier_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_parameter_doc,
"set_parameter_rcl(pname,pvalue)\n\n\
Sets value for the parameter with name `pname` to `pvalue`.\n\
Note that `imag` is supported, but is not tested.\n\
pname  = string  -> internal parameter name\n\
pvalue = complex -> parameter value");
static PyObject *
set_parameter_py(PyObject *self, PyObject *args)
{
  const char* pname;
  Py_complex pvalue;

  if (!PyArg_ParseTuple(args, "sD", &pname, &pvalue))
      return NULL;

  dcomplex pvaluec;
  pvaluec.dr = pvalue.real;
  pvaluec.di = pvalue.imag;

  set_parameter_rcl(pname, pvaluec);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(get_parameter_doc,
"get_parameter_rcl(pname)\n\n\
Gets value for the parameter with name `pname`.\n\
pname  = string       -> parameter name\n\
return = float, float -> real part, imaginary part");
static PyObject *
get_parameter_py(PyObject *self, PyObject *args)
{
  const char* pname;

  if (!PyArg_ParseTuple(args, "s", &pname))
      return NULL;

  dcomplex pvaluec;
  get_parameter_rcl(pname, &pvaluec);

  return Py_BuildValue("dd", pvaluec.dr, pvaluec.di);
}
PyDoc_STRVAR(set_renoscheme_doc,
"set_renoscheme_rcl(rname,rscheme)\n\n\
Sets the renormalization scheme for the parameter `rname` to `rscheme`.\n\
rname   = string -> internal ct parameter name\n\
rscheme = string -> internal renormalization scheme name");
static PyObject *
set_renoscheme_py(PyObject *self, PyObject *args)
{
  const char* rname;
  const char* rscheme;

  if (!PyArg_ParseTuple(args, "ss", &rname, &rscheme))
      return NULL;

  set_renoscheme_rcl(rname, rscheme);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_resonant_particle_doc,
"set_resonant_particle_rcl(pname)\n\n\
Tags the particle with name `pname` as resonant.\n\
pname = string -> internal name of the partice");
static PyObject *
set_resonant_particle_py(PyObject *self, PyObject *args)
{
  const char* p;

  if (!PyArg_ParseTuple(args, "s", &p))
      return NULL;

  set_resonant_particle_rcl(p);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_quarkline_doc,
"set_quarkline_rcl(npr,q1,q2)\n\n\
Enforces a quark-flow between two quarks. Note that all quark pairs need\n\
a quark-flow assigned for this function to work as expected.\n\
npr = integer -> process number\n\
q1 = integer -> 1st quark position\n\
q2 = integer -> 2st quark position ");
static PyObject *
set_quarkline_py(PyObject *self, PyObject *args)
{
  int npr,q1,q2;

  if (!PyArg_ParseTuple(args, "iii", &npr,&q1,&q2))
      return NULL;

  set_quarkline_rcl(npr,q1,q2);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(reset_vertices_doc,
"reset_vertices_rcl\n\n\
Resets the vertices, couplings and counterterm couplings in the model.");
static PyObject *
reset_vertices_py(PyObject *self, PyObject *args)
{

  reset_vertices_rcl();

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(reset_couplings_doc,
"reset_couplings_rcl\n\n\
Resets the couplings and counterterm couplings in the model.");
static PyObject *
reset_couplings_py(PyObject *self, PyObject *args)
{

  reset_couplings_rcl();

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(reset_ctcouplings_doc,
"reset_ctcouplings_rcl\n\n\
Resets the counterterm couplings in the model.");
static PyObject *
reset_ctcouplings_py(PyObject *self, PyObject *args)
{

  reset_ctcouplings_rcl();

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(print_collier_statistics_doc,
"print_collier_statistics_rcl\n\n\
Prints the collier statistics to the currently opened output channel.");
static PyObject *
print_collier_statistics_py(PyObject *self, PyObject *args)
{

  print_collier_statistics_rcl();

  Py_INCREF(Py_None);
  return Py_None; 
}

// additional stuff, only used in combination with rept1l
static PyObject *
set_compute_selfenergy_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_compute_selfenergy_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_compute_selfenergy_offshell_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_compute_selfenergy_offshell_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_compute_tadpole_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_compute_tadpole_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_lp_py(PyObject *self, PyObject *args)
{
  int lp_order;
  int value;

  if (!PyArg_ParseTuple(args, "ii", &lp_order, &value))
      return NULL;

  set_lp_rcl(lp_order, value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_form_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_form_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_all_couplings_active_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_all_couplings_active_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_all_tree_couplings_active_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_all_tree_couplings_active_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_all_loop_couplings_active_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_all_loop_couplings_active_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_all_ct_couplings_active_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_all_ct_couplings_active_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_all_r2_couplings_active_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_all_r2_couplings_active_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_complex_mass_form_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_complex_mass_form_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_vertex_functions_py(PyObject *self, PyObject *args)
{
  int value;
  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_vertex_functions_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
disable_particle_ordering_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  disable_particle_ordering_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_masscut_form_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_masscut_form_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_cutparticle_py(PyObject *self, PyObject *args)
{
  const char* particle_str;

  if (!PyArg_ParseTuple(args, "s", &particle_str))
      return NULL;
  set_cutparticle_rcl(particle_str);
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
unset_cutparticle_py(PyObject *self, PyObject *args)
{
  unset_cutparticle_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *
set_print_recola_logo_py(PyObject *self, PyObject *args)
{
  int value;

  if (!PyArg_ParseTuple(args, "i", &value))
      return NULL;

  set_print_recola_logo_rcl(value);
  Py_INCREF(Py_None);
  return Py_None;
}

/**********************************
*  module recola1_interface_rcl  *
**********************************/

PyDoc_STRVAR(set_pole_mass_w_doc,
"set_pole_mass_w_rcl(m,g)\n\n\
Sets the pole mass and width of the W boson.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_w_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_w_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_onshell_mass_w_doc,
"set_onshell_mass_w_rcl(m,g)\n\n\
Sets the on-shell mass and width of the W boson.\n\
m = float -> on-shell mass \n\
w = float -> on-shell width");
static PyObject *
set_onshell_mass_w_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_onshell_mass_w_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_z_doc,
"set_pole_mass_z_rcl(m,g)\n\n\
Sets the pole mass and width of the Z boson.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_z_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_z_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_onshell_mass_z_doc,
"set_onshell_mass_z_rcl(m,g)\n\n\
Sets the on-shell mass and width of the Z boson.\n\
m = float -> on-shell mass \n\
w = float -> on-shell width");
static PyObject *
set_onshell_mass_z_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_onshell_mass_z_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_h_doc,
"set_pole_mass_h_rcl(m,g)\n\n\
Sets the pole mass and width of the Higgs boson.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_h_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_h_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_top_doc,
"set_pole_mass_top_rcl(m,g)\n\n\
Sets the pole mass and width of the top-quark.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_top_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_top_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_bottom_doc,
"set_pole_mass_bottom_rcl(m,g)\n\n\
Sets the pole mass and width of the bottom-quark.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_bottom_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_bottom_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_charm_doc,
"set_pole_mass_charm_rcl(m,g)\n\n\
Sets the pole mass and width of the charm-quark.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_charm_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_charm_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_strange_doc,
"set_pole_mass_strange_rcl(m)\n\n\
Sets the pole mass of the strange-quark.\n\
m = float -> pole mass");
static PyObject *
set_pole_mass_strange_py(PyObject *self, PyObject *args)
{

  double m;
  if (!PyArg_ParseTuple(args, "d", &m))
      return NULL;

  set_pole_mass_strange_rcl(m);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_up_doc,
"set_pole_mass_up_rcl(m)\n\n\
Sets the pole mass of the up-quark.\n\
m = float -> pole mass");
static PyObject *
set_pole_mass_up_py(PyObject *self, PyObject *args)
{

  double m;
  if (!PyArg_ParseTuple(args, "d", &m))
      return NULL;

  set_pole_mass_up_rcl(m);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_down_doc,
"set_pole_mass_down_rcl(m)\n\n\
Sets the pole mass of the down-quark.\n\
m = float -> pole mass");
static PyObject *
set_pole_mass_down_py(PyObject *self, PyObject *args)
{

  double m;
  if (!PyArg_ParseTuple(args, "d", &m))
      return NULL;

  set_pole_mass_down_rcl(m);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_tau_doc,
"set_pole_mass_tau_rcl(m,g)\n\n\
Sets the pole mass and width of the tau.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_tau_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_tau_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_muon_doc,
"set_pole_mass_muon_rcl(m,g)\n\n\
Sets the pole mass and width of the muon.\n\
m = float -> pole mass \n\
w = float -> pole width");
static PyObject *
set_pole_mass_muon_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_muon_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_electron_doc,
"set_pole_mass_electron_rcl(m)\n\n\
Sets the pole mass of the electron.\n\
m = float -> pole mass");
static PyObject *
set_pole_mass_electron_py(PyObject *self, PyObject *args)
{

  double m;
  if (!PyArg_ParseTuple(args, "d", &m))
      return NULL;

  set_pole_mass_electron_rcl(m);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_light_top_doc,
"set_light_top_rcl\n\n\
Sets the top-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_top_py(PyObject *self, PyObject *args)
{
  set_light_top_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_top_doc,
"unset_light_top_rcl\n\n\
unsets the top-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_top_py(PyObject *self, PyObject *args)
{
  unset_light_top_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_bottom_doc,
"set_light_bottom_rcl\n\n\
Sets the bottom-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_bottom_py(PyObject *self, PyObject *args)
{
  set_light_bottom_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_bottom_doc,
"unset_light_bottom_rcl\n\n\
unsets the bottom-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_bottom_py(PyObject *self, PyObject *args)
{
  unset_light_bottom_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_charm_doc,
"set_light_charm_rcl\n\n\
Sets the charm-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_charm_py(PyObject *self, PyObject *args)
{
  set_light_charm_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_charm_doc,
"unset_light_charm_rcl\n\n\
unsets the charm-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_charm_py(PyObject *self, PyObject *args)
{
  unset_light_charm_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_strange_doc,
"set_light_strange_rcl\n\n\
Sets the strange-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_strange_py(PyObject *self, PyObject *args)
{
  set_light_strange_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_strange_doc,
"unset_light_strange_rcl\n\n\
unsets the strange-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_strange_py(PyObject *self, PyObject *args)
{
  unset_light_strange_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_up_doc,
"set_light_up_rcl\n\n\
Sets the up-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_up_py(PyObject *self, PyObject *args)
{
  set_light_up_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_up_doc,
"unset_light_up_rcl\n\n\
unsets the up-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_up_py(PyObject *self, PyObject *args)
{
  unset_light_up_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_down_doc,
"set_light_down_rcl\n\n\
Sets the down-quark as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_down_py(PyObject *self, PyObject *args)
{
  set_light_down_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_down_doc,
"unset_light_down_rcl\n\n\
unsets the down-quark as light, keeping the full mass dependence.");
static PyObject *
unset_light_down_py(PyObject *self, PyObject *args)
{
  unset_light_down_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_tau_doc,
"set_light_tau_rcl\n\n\
Sets the tau as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_tau_py(PyObject *self, PyObject *args)
{
  set_light_tau_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_tau_doc,
"unset_light_tau_rcl\n\n\
unsets the tau as light, keeping the full mass dependence.");
static PyObject *
unset_light_tau_py(PyObject *self, PyObject *args)
{
  unset_light_tau_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_muon_doc,
"set_light_muon_rcl\n\n\
Sets the muon as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_muon_py(PyObject *self, PyObject *args)
{
  set_light_muon_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_muon_doc,
"unset_light_muon_rcl\n\n\
unsets the muon as light, keeping the full mass dependence.");
static PyObject *
unset_light_muon_py(PyObject *self, PyObject *args)
{
  unset_light_muon_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_electron_doc,
"set_light_electron_rcl\n\n\
Sets the electron as light, regularizing IR singularities with its mass.");
static PyObject *
set_light_electron_py(PyObject *self, PyObject *args)
{
  set_light_electron_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unset_light_electron_doc,
"unset_light_electron_rcl\n\n\
unsets the electron as light, keeping the full mass dependence.");
static PyObject *
unset_light_electron_py(PyObject *self, PyObject *args)
{
  unset_light_electron_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_light_fermions_doc,
"set_light_fermions_rcl(mcut)\n\n\
Sets a masscut `mcut` below which all massive fermions are treated\n\
as light particles, i.e. their collinear singularities are regularized with\n\
their mass and terms proportional to the mass are discarded).\n\
Massive fermions with a mass larger than `mcut` are heavy, and their\n\
full mass dependence is kept.\n\
mcut = float -> masscut value\n\n\
This function has no effect on massless\n\
fermions (their collinear singularities are regularized dimensionally).\n\
mcut = positiveflotating point number -> mass cut in GeV");
static PyObject *
set_light_fermions_py(PyObject *self, PyObject *args)
{

  double mcut;
  if (!PyArg_ParseTuple(args, "d", &mcut))
      return NULL;

  set_light_fermions_rcl(mcut);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_gfermi_scheme_doc,
"use_gfermi_scheme_rcl(g=None,a=None)\n\n\
Sets the EW renormalization scheme to the gfermi scheme.\n\
g = float -> Sets the value of the Fermi constant to `g`\n\
a = float -> Sets value of alpha_gf to `a`\n\
Note that either g or a can be non-zero, but not both at the same time.\n\
For `g=a=None` (DEFAULT) the hard-coded value for alpha is used.");
static PyObject *
use_gfermi_scheme_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  PyObject *a_py = Py_None;
  PyObject *g_py = Py_None;
  static char *kwlist[] = {"g", "a", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "|OO", kwlist, &g_py, &a_py))
      return NULL;

  if (g_py != Py_None && a_py == Py_None) {
    double g = PyFloat_AsDouble(g_py);
    use_gfermi_scheme_and_set_gfermi_rcl(g);
  } else if (g_py == Py_None && a_py != Py_None) {
    double a = PyFloat_AsDouble(a_py);
    use_gfermi_scheme_and_set_alpha_rcl(a);
  } else if (g_py == Py_None && a_py == Py_None){
    use_gfermi_scheme_rcl();
  } else{
    PyErr_SetString(PyExc_Exception, "Cannot pass g and a to use_gfermi_scheme_rcl!");
    return (PyObject *) NULL;
  }

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_alpha0_scheme_doc,
"use_alpha0_scheme_rcl(a=None)\n\n\
Sets the EW renormalization scheme to the alpha0 scheme.\n\
a = float -> Sets value of alpha to `a`\n\
For `a=None` (DEFAULT) the hard-coded value for alpha is used.");
static PyObject *
use_alpha0_scheme_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  PyObject *a_py = Py_None;
  static char *kwlist[] = {"a", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "|O", kwlist, &a_py))
      return NULL;

  if (a_py != Py_None) {
    if (!PyFloat_Check(a_py)) {
      // error, we are expecting an float
      PyErr_SetString(PyExc_Exception, "a is not of type float.");
      return (PyObject *) NULL;
    }
    double a = PyFloat_AsDouble(a_py);
    use_alpha0_scheme_and_set_alpha_rcl(a);
  } else {
    use_alpha0_scheme_rcl();
  }

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_alphaZ_scheme_doc,
"use_alphaZ_scheme_rcl(a=None)\n\n\
Sets the EW renormalization scheme to the alphaZ scheme.\n\
a = float -> Sets value of alpha to `a`\n\
For `a=None` (DEFAULT) the hard-coded value for alpha is used.");
static PyObject *
use_alphaZ_scheme_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  PyObject *a_py = Py_None;
  static char *kwlist[] = {"a", NULL};

  if (!PyArg_ParseTupleAndKeywords(args, keywds, "|O", kwlist, &a_py))
      return NULL;

  if (a_py != Py_None) {
    if (!PyFloat_Check(a_py)) {
      // error, we are expecting an float
      PyErr_SetString(PyExc_Exception, "a is not of type float.");
      return (PyObject *) NULL;
    }
    double a = PyFloat_AsDouble(a_py);
    use_alphaz_scheme_and_set_alpha_rcl(a);
  } else {
    use_alphaz_scheme_rcl();
  }

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(get_alpha_doc,
"get_alpha_rcl()\n\n\
Return the value for the parameter `alpha`.\n\
return = alpha");
static PyObject *
get_alpha_py(PyObject *self, PyObject *args)
{
  double value;
  get_alpha_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(set_alphas_doc,
"set_alphas_rcl(as,Q,Nf)\n\n\
Sets the values of `alpha_s` to `as`, the renormalization scale\n\
`Qren`(muMS) to `s` and the number of active quark flavours `Nfren`\n\
to `Nf`. The choosen value of `as` should correspond to the physical value\n\
of alpha_s at the scale `s` computed in the `Nf`-flavour scheme.\n\
Accepted values for `Nf` are -1,3,4,5,6\n\
as = float -> value of `alpha_s`\n\
Q  = float -> renormalization scale\n\
Nf = -1      -> all quarks with mass less than `s` are active flavours\n\
Nf = 3,4,5,6 -> the 3,4,5,6 lightest quarks are the active flavours");
static PyObject *
set_alphas_py(PyObject *self, PyObject *args)
{

  double aS, scaleQCD;
  int nflavour;

  if (!PyArg_ParseTuple(args, "ddi", &aS, &scaleQCD, &nflavour))
      return NULL;

  set_alphas_rcl(&aS, &scaleQCD, &nflavour);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(get_alphas_doc,
"get_alphas_rcl()\n\n\
Return the value for the parameter `alpha_s`.");
static PyObject *
get_alphas_py(PyObject *self, PyObject *args)
{
  double value;
  get_alphas_rcl(&value);
  return Py_BuildValue("d", value);
}
PyDoc_STRVAR(set_alphas_masses_doc,
"set_alphas_masses_rcl(mc,mb,mt,gc=0.,gb=0.,gt=0.)\n\n\
Overwrites the values of the quark masses for the running of alpha_s.\n\
Those values are not used in the rest of the computation.\n\
The squared masses of the 3 heaviest quarks are set to:\n\
`mc^2-I*mc*gc`, `mb^2-I*mb*gb` and `mt^2-I*mt*gt`, respectively.\n\
`mc`, `mb`, `mt` must satisfy 0 <= mc <= mb <= mt.\n\
The arguments `gc`, `gb`, `gt` are optional and can be left to zero.");
static PyObject *
set_alphas_masses_py(PyObject *self, PyObject *args, PyObject *keywds)
{

  double mc, mb, mt, gc, gb, gt;
  gc = 0.;
  gb = 0.;
  gt = 0.;

  static char *kwlist[] = {"mc", "mb", "mt", "gc", "gb", "gt", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "ddd|ddd", kwlist,
                                   &mc, &mb, &mt, &gc, &gb, &gt))
      return NULL;

  set_alphas_masses_rcl(mc, mb, mt, gc, gb, gt);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(compute_running_alphas_doc,
"compute_running_alphas_rcl(Q,Nf,lp)\n\n\
Computes the value for alpha_s at the scale `Q` employing the \n\
renormalization-group evolution at `lp` loops\n\
(`lp` can take the value `lp=1` or `lp=2`).\n\
The integer argument `Nf` selects the flavour scheme according to\n\
the following rules:\n\
Q  = float -> renormalization scale\n\
Nf = -1  -> The variable flavour scheme is selected, where the number of\n\
            active flavours contributing to the running of alpha_s is set\n\
            to the number of quarks lighter than the scale `Q`.\n\
Nf = 3-6 -> The fixed Nf-flavour scheme is selected, where the number of\n\
            active flavours contributing to the running of alpha_s is set\n\
            to `Nf`. In this case `Nf` cannot be smaller than the number of\n\
            massless quarks (otherwise the code stops).\n\
lp = integer -> loop-order, accepted values: 1,2");
static PyObject *
compute_running_alphas_py(PyObject *self, PyObject *args)
{
  double Q;
  int Nf,lp;

  if (!PyArg_ParseTuple(args, "dii", &Q, &Nf, &lp))
      return NULL;

  compute_running_alphas_rcl(Q, Nf, lp);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(select_all_gs_powers_BornAmpl_doc,
"select_all_gs_powers_BornAmpl_rcl(npr)\n\n\
Selects all the contribution to the Born amplitude for the process\n\
with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
select_all_gs_powers_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  select_all_gs_powers_BornAmpl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_all_gs_powers_BornAmpl_doc,
"unselect_all_gs_powers_BornAmpl_rcl(npr)\n\n\
Unselects all the contribution to the Born amplitude for the process\n\
with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
unselect_all_gs_powers_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  unselect_all_gs_powers_BornAmpl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_gs_power_BornAmpl_doc,
"select_gs_power_BornAmpl_rcl(npr,gspower)\n\n\
Selects the contribution to the Born amplitude with g_s power `gspower`,\n\
for the process with process number `npr`.\n\
npr     = integer -> process number\n\
gspower = integer -> power in gs");
static PyObject *
select_gs_power_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id, gspower;

  if (!PyArg_ParseTuple(args, "ii", &process_id, &gspower))
    return NULL;

  select_gs_power_BornAmpl_rcl(process_id, gspower);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_gs_power_BornAmpl_doc,
"unselect_gs_power_BornAmpl_rcl(npr,gspower)\n\n\
Unselects the contribution to the Born amplitude with g_s power `gspower`,\n\
for the process with process number `npr`.\n\
npr     = integer -> process number\n\
gspower = integer -> power in gs");
static PyObject *
unselect_gs_power_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id, gspower;

  if (!PyArg_ParseTuple(args, "ii", &process_id, &gspower))
    return NULL;

  unselect_gs_power_BornAmpl_rcl(process_id, gspower);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_all_gs_powers_LoopAmpl_doc,
"select_all_gs_powers_LoopAmpl_rcl(npr)\n\n\
Selects all the contribution to the Born amplitude for the process\n\
with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
select_all_gs_powers_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  select_all_gs_powers_LoopAmpl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_all_gs_powers_LoopAmpl_doc,
"unselect_all_gs_powers_LoopAmpl_rcl(npr)\n\n\
Unselects all the contribution to the Born amplitude for the process\n\
with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
unselect_all_gs_powers_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  unselect_all_gs_powers_LoopAmpl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_gs_power_LoopAmpl_doc,
"select_gs_power_LoopAmpl_rcl(npr, gspower)\n\n\
Selects the contribution to the Loop amplitude with g_s power `gspower`,\n\
for the process with process number `npr`.\n\
npr     = integer -> process number\n\
gspower = integer -> power in gs");
static PyObject *
select_gs_power_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id, gspower;

  if (!PyArg_ParseTuple(args, "ii", &process_id, &gspower))
    return NULL;

  select_gs_power_LoopAmpl_rcl(process_id, gspower);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_gs_power_LoopAmpl_doc,
"unselect_gs_power_LoopAmpl_rcl(npr,gspower)\n\n\
Unselects the contribution to the Loop amplitude with g_s power `gspower`,\n\
for the process with process number `npr`.\n\
npr     = integer -> process number\n\
gspower = integer -> power in gs");
static PyObject *
unselect_gs_power_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id, gspower;

  if (!PyArg_ParseTuple(args, "ii", &process_id, &gspower))
    return NULL;

  unselect_gs_power_LoopAmpl_rcl(process_id, gspower);
  Py_INCREF(Py_None);
  return Py_None;
}

/*******************************
*  module process definition  *
*******************************/

PyDoc_STRVAR(define_process_doc,
"define_process_rcl(npr,process,order)\n\n\
Define a process by providing a process id `npr` of type integer,\n\
a process string `process` and the (loop-)order of type string:\n\
npr     = integer -> process number\n\
process = string  -> Process string composed of particles and\n\
                     separated by ' -> '\n\
order   = string  -> (loop-) order, accepted values ’LO’ and ’NLO’\n\n\
Issuing repeated calls of define_process_rcl allows to define more than one\n\
process. Each process must be given a unique identifier npr.\n\
The process string must consist of a list of particles separated by ’ -> ’,\n\
with the incoming (outgoing) particles on the left-hand (right-hand) side\n\
of ’ -> ’. Any number of incoming and outgoing particles is allowed.\n\
The symbols for the particles, listed in Section 2.9, as well as the symbol\n\
’->’ must be separated by at least one blank character. Examples of allowed\n\
character chains are:\n\
  ’e+ e- -> mu+ mu-’,\n\
  ’H -> A A’,\n\
  ’u d~ -> W+ g  g g’,\n\
  ’u g -> u g Z’, \n\
  ’u  g  -> u   g  tau-    tau+’ .\n\n\
To each fermion or vector particle a specific helicity can be attributed by\n\
adding the corresponding symbol ’[-]’, ’[+]’ or ’[0]’. Blank characters can\n\
(but need not) separate the helicity symbol from the particle name:\n\
  ’e+[+] e- [-] -> Z H’: ’e+’ is right-handed, ’e-’ is left-handed,\n\
                         ’Z’ is unpolarized.\n\
  ’u u~ -> W+[-] W-[+]’: ’u’ and ’u~’ are unpolarized,\n\
                         ’W+’ and ’W-’ are transverse.\n\n\
Contributions with specific intermediate states can be selected\n\
where intermediate states are particles decaying into any number of other\n\
particles. To this end, in the process declaration the decaying particle must\n\
be followed by round brackets ’( ... )’ containing the decay products.\n\
Multiple and nested decays are allowed. Blank characters can (but need not)\n\
separate the brackets ’(’, ’)’ from the particle names:\n\
  ’e+ e- -> W+ W-(e- nu_e~)’,\n\
  ’e+ e- -> Z H ( b~[+] b[-] )’,\n\
  ’e+ e- -> t( W+(u d~) b) t~(e- nu_e~ b~)’,\n\
  ’e+ e- -> Z ( mu+(e+ nu_e nu_mu~) mu-(e- nu_e~ nu_mu) ) H’ .\n\
The selection of specific intermediate particles is a prerequisite for the\n\
calculation of amplitudes within the pole approximation.");
static PyObject *
define_process_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* process;
  const char* order;

  if (!PyArg_ParseTuple(args, "iss", &process_id, &process, &order))
      return NULL;

  define_process_rcl(process_id, process, order);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_power_BornAmpl_doc,
"select_power_BornAmpl_rcl(npr,cid,pow)\n\n\
Selects for the process with process number `npr` the contribution to\n\
the Born amplitude with power `power` in the fundamental order `cid`.\n\
npr = integer -> process number\n\
cid = string  -> fundamental coupling id\n\
pow = integer -> power in fundamental coupling\n\n\
E.g. the selection of the Born contribution e^4 g_s^2:\n\
select_power_BornAmpl_rcl(npr, 'QCD', 2)\n\
select_power_BornAmpl_rcl(npr, 'QED', 4)");
static PyObject *
select_power_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* order;
  int power;

  if (!PyArg_ParseTuple(args, "isi", &process_id, &order, &power))
    return NULL;

  select_power_bornampl_rcl(process_id, order, power);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_power_BornAmpl_doc,
"unselect_power_BornAmpl_rcl(npr,cid,pow)\n\
Selects the contributions to the Born amplitude with power `power` of \n\
the fundanmetal order `cid` for the process with identifier `npr`.\n\
npr = integer -> process number\n\
cid = string  -> fundamental coupling id\n\
pow = integer -> power in fundamental coupling");
static PyObject *
unselect_power_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* order;
  int power;

  if (!PyArg_ParseTuple(args, "isi", &process_id, &order, &power))
    return NULL;

  unselect_power_bornampl_rcl(process_id, order, power);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_all_powers_BornAmpl_doc,
"select_all_powers_BornAmpl_rcl(npr)\n\
Selects all contributions to the Born amplitude with any power of orders.\n\
npr = integer -> process number");
static PyObject *
select_all_powers_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  select_all_powers_bornampl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_all_powers_BornAmpl_doc,
"unselect_all_powers_BornAmpl_rcl()\n\
Unselects all contributions to the Born amplitude with any power of orders.");
static PyObject *
unselect_all_powers_BornAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  unselect_all_powers_bornampl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_power_LoopAmpl_doc,
"select_power_LoopAmpl_rcl(npr,cid,pow)\n\n\
Selects for the process with process number `npr` the contribution to\n\
the Loop amplitude with power `power` in the fundamental order `cid`.\n\
npr = integer -> process number\n\
cid = string  -> fundamental coupling id\n\
pow = integer -> power in fundamental coupling\n\n\
E.g. the selection of the Loop contribution e^4 g_s^2:\n\
select_power_LoopAmpl_rcl(npr, 'QCD', 2)\n\
select_power_LoopAmpl_rcl(npr, 'QED', 4)");
static PyObject *
select_power_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* order;
  int power;

  if (!PyArg_ParseTuple(args, "isi", &process_id, &order, &power))
    return NULL;

  select_power_loopampl_rcl(process_id, order, power);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_power_LoopAmpl_doc,
"unselect_power_LoopAmpl_rcl()\n\
Selects all contributions to the Loop amplitude with any power of orders.");
static PyObject *
unselect_power_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* order;
  int power;

  if (!PyArg_ParseTuple(args, "isi", &process_id, &order, &power))
    return NULL;

  unselect_power_loopampl_rcl(process_id, order, power);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(select_all_powers_LoopAmpl_doc,
"select_all_powers_LoopAmpl()\n\
Selects all contributions to the Loop amplitude with any power of orders.");
static PyObject *
select_all_powers_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  select_all_powers_loopampl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(unselect_all_powers_LoopAmpl_doc,
"unselect_all_powers_LoopAmpl()\n\
Unselects all contributions to the Loop amplitude with any power of orders.");
static PyObject *
unselect_all_powers_LoopAmpl_py(PyObject *self, PyObject *args)
{
  int process_id;

  if (!PyArg_ParseTuple(args, "i", &process_id))
    return NULL;

  unselect_all_powers_loopampl_rcl(process_id);
  Py_INCREF(Py_None);
  return Py_None;
}

PyDoc_STRVAR(split_collier_cache_doc,
"set_collier_cache_rcl(npr, n)\n\n\
Splits the cache of collier for the process `npr` in `n` parts.\n\
npr = integer -> process number\n\
n   = integer -> number of caches");
static PyObject *
split_collier_cache_py(PyObject *self, PyObject *args)
{
  int process_id,n;

  if (!PyArg_ParseTuple(args, "ii", &process_id, &n))
    return NULL;

  split_collier_cache_rcl(process_id, n);
  Py_INCREF(Py_None);
  return Py_None;
}

/***********************************
*  module process_generation_rcl  *
***********************************/

PyDoc_STRVAR(generate_processes_doc,
"generate_processes_rcl()\n\n\
Generates all defined processes.");
static PyObject *
generate_processes_py(PyObject *self, PyObject *args)
{
  generate_processes_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}

/************************************
*  module process_computation_rcl  *
************************************/

PyDoc_STRVAR(set_resonant_squared_momentum_doc,
"set_resonant_squared_momentum_rcl(npr,res,ps)\n\n\
Sets for the resonance with resonance number `res` for the process\n\
with process number `npr` the squared momentum of the denominator\n\
of the resonant propagator to `ps`.\n\
npr = integer -> process number\n\
res = integer -> resonance numer, the first resonant particle defined\n\
                 has `res=1`, the second resonant particle has `res=2`,\n\
                 and so on\n\
ps  = float   -> squared momentum of the resonant propagator");
static PyObject *
set_resonant_squared_momentum_py(PyObject *self, PyObject *args)
{

  int npr,res;
  double ps;

  if (!PyArg_ParseTuple(args, "iid", &npr, &res, &ps))
      return NULL;

  set_resonant_squared_momentum_rcl(npr, res, ps);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(rescale_process_doc,
"rescale_process_rcl(npr,order)\n\n\
Adjusts the results calculated by compute_process_rcl for a new value\n\
of alpha_s, rescaling the structure-dressed helicity amplitudes and\n\
recomputing the summed squared amplitude for the process with process\n\
number `npr`.\n\
npr    = integer -> process number\n\
order  = string  -> either 'LO' or 'NLO'\n\
return = (A^2_0, A^2_1)");
static PyObject *
rescale_process_py(PyObject *self, PyObject *args)
{

  int npr;
  const char* order;

  if (!PyArg_ParseTuple(args, "is", &npr, &order))
      return NULL;

  double A2[2];
  rescale_process_rcl(npr, order, A2);

  return Py_BuildValue("dd", A2[0], A2[1]);
}
PyDoc_STRVAR(rescale_colour_correlation_doc,
"rescale_colour_correlation_rcl(npr,i1,i2,order='LO')\n\n\
Adjusts the results calculated by compute_colour_correlation_rcl for a\n\
new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the colour-correlated summed squared amplitude for the\n\
process with process number `npr`.\n\
npr    = integer -> process number\n\
i1     = integer -> quark/anti-quark at leg i1\n\
i2     = integer -> quark/anti-quark at leg i2\n\
order  = string  -> either 'LO' (default) or 'NLO'\n\
return = A^2_{cc}");
static PyObject *
rescale_colour_correlation_py(PyObject *self, PyObject *args, PyObject *keywds)
{

  int npr,i1,i2;
  PyObject *order_py;
  order_py = Py_None;
  static char *kwlist[] = {"npr", "i1", "i2", "order", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "iii|O", kwlist,
                                   &npr, &i1, &i2,
                                   &order_py))
      return NULL;

  const char *order;
  if (order_py != Py_None) {
    order = pyobj_to_string(order_py);
  } else {
    order = "LO";
  }

  double A2cc;
  rescale_colour_correlation_rcl(npr, i1, i2, order, &A2cc);

  return Py_BuildValue("d", A2cc);
}
PyDoc_STRVAR(rescale_colour_correlation_int_doc,
"rescale_colour_correlation_int_rcl(npr,i1,i2,)\n\n\
Adjusts the results calculated by compute_colour_correlation_int_rcl for a\n\
new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the colour-correlated summed 1-loop interference amplitude for\n\
the process with process number `npr`.\n\
npr = integer -> process number\n\
i1  = integer -> quark/anti-quark at leg i1\n\
i2  = integer -> quark/anti-quark at leg i2\n\
return = A^2_{cc}");
static PyObject *
rescale_colour_correlation_int_py(PyObject *self, PyObject *args)
{

  int npr,i1,i2;

  if (!PyArg_ParseTuple(args, "iii", &npr, &i1, &i2))
      return NULL;

  double A2ccint;
  rescale_colour_correlation_int_rcl(npr, i1, i2, &A2ccint);

  return Py_BuildValue("d", A2ccint);
}
PyDoc_STRVAR(rescale_all_colour_correlations_doc,
"rescale_all_colour_correlations_rcl(npr)\n\n\
Adjusts the results calculated by compute_all_colour_correlation_rcl for\n\
a new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the colour-correlated summed squared amplitude for the\n\
process with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
rescale_all_colour_correlations_py(PyObject *self, PyObject *args)
{

  int npr;

  if (!PyArg_ParseTuple(args, "i", &npr))
      return NULL;

  double A2cc;
  rescale_all_colour_correlations_rcl(npr);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(rescale_all_colour_correlations_int_doc,
"rescale_all_colour_correlations_int_rcl(npr)\n\n\
Adjusts the results calculated by compute_all_colour_correlation_rcl for\n\
a new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the colour-correlated summed squared amplitude for the\n\
process with process number `npr`.\n\
npr = integer -> process number");
static PyObject *
rescale_all_colour_correlations_int_py(PyObject *self, PyObject *args)
{

  int npr;

  if (!PyArg_ParseTuple(args, "i", &npr))
      return NULL;

  double A2cc;
  rescale_all_colour_correlations_int_rcl(npr);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(rescale_spin_colour_correlation_doc,
"rescale_spin_colour_correlation_rcl(npr,i1,i2,v)\n\n\
Adjusts the results calculated by compute_spin_colour_correlation_rcl for\n\
a new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the spin-colour-correlated summed squared amplitude for the\n\
process with process number `npr`.\n\
npr = integer    -> process number\n\
i1  = integer    -> gluon at leg i1\n\
i2  = integer    -> quark/anti-quark at leg i2\n\
v   = complex[4] -> polarization vector\n\
return = A^2_{scc}");
static PyObject *
rescale_spin_colour_correlation_py(PyObject *self, PyObject *args)
{

  int npr,i1,i2;
  PyObject *polvec_py;

  if (!PyArg_ParseTuple(args, "iiiO!", &npr, &i1, &i2, &PyList_Type, &polvec_py))
      return NULL;

  dcomplex v[4];
  if (!build_c_polvec(polvec_py, v))
  {
    return (PyObject *) NULL;
  }

  double A2scc;
  rescale_spin_colour_correlation_rcl(npr, i1, i2, v, &A2scc);

  return Py_BuildValue("d", A2scc);
}
PyDoc_STRVAR(rescale_spin_correlation_doc,
"rescale_spin_correlation_rcl(npr,j,v)\n\n\
Adjusts the results calculated by compute_spin_correlation_rcl for a\n\
new value of alpha_s, rescaling the structure-dressed helicity amplitudes\n\
and recomputing the spin-correlated summed squared amplitude for the\n\
process with process number `npr`.\n\
npr = integer    -> process number\n\
j   = integer    -> photon/gluon leg\n\
v   = complex[4] -> photon/gluon polarization vector\n\
return = A2sc");
static PyObject *
rescale_spin_correlation_py(PyObject *self, PyObject *args)
{

  int npr,j;
  PyObject *polvec_py;

  if (!PyArg_ParseTuple(args, "iiiO!", &npr, &j, &PyList_Type, &polvec_py))
      return NULL;

  dcomplex v[4];
  if (!build_c_polvec(polvec_py, v))
  {
    return (PyObject *) NULL;
  }

  double A2scc;
  rescale_spin_correlation_rcl(npr, j, v, &A2scc);

  return Py_BuildValue("d", A2scc);
}
PyDoc_STRVAR(set_tis_required_accuracy_doc,
"set_tis_required_accuracy_rcl(acc)\n\n\
Sets the required accuracy for TIs to the value `acc` in collier.\n\
acc = floating point number -> accarcy in 10^-DIGITS, e.g. 0.00001.");
static PyObject *
set_tis_required_accuracy_py(PyObject *self, PyObject *args)
{
  double acc;

  if (!PyArg_ParseTuple(args, "d", &acc))
    return NULL;

  set_tis_required_accuracy_rcl(acc);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_tis_required_accuracy_doc,
"get_tis_required_accuracy_rcl()\n\n\
Gets the required accuracy set for TIs\n\
return = floating point number.");
static PyObject *
get_tis_required_accuracy_py(PyObject *self, PyObject *args)
{

  double acc;
  get_tis_required_accuracy_rcl(&acc);

  return Py_BuildValue("d", acc);
}
PyDoc_STRVAR(set_tis_critical_accuracy_doc,
"set_tis_critical_accuracy_rcl(acc)\n\n\
Sets the critical accuracy for TIs to the value `acc` in collier.\n\
acc = floating point number -> accarcy in 10^-DIGITS, e.g. 0.00001.");
static PyObject *
set_tis_critical_accuracy_py(PyObject *self, PyObject *args)
{
  double acc;

  if (!PyArg_ParseTuple(args, "d", &acc))
    return NULL;

  set_tis_critical_accuracy_rcl(acc);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(get_tis_critical_accuracy_doc,
"get_tis_critical_accuracy_rcl()\n\n\
Gets the critical accuracy set for TIs\n\
return = floating point number.");
static PyObject *
get_tis_critical_accuracy_py(PyObject *self, PyObject *args)
{

  double acc;
  get_tis_critical_accuracy_rcl(&acc);

  return Py_BuildValue("d", acc);
}
PyDoc_STRVAR(get_tis_accuracy_flag_doc,
"get_tis_accuracy_flag_rcl()\n\n\
Gets the value of the accuracy flag for TIs from collier. The return\n\
value contains global information on the accuracy of the TIs evaluated in\n\
the last call of compute_process_rcl:\n\
return =  0 -> For all TIs, the accuracy is estimated to be better than the\n\
               required value.\n\
return = -1 -> For at least one TI, the accuracy is estimated to be worse\n\
               than the required value, but for all TIs, the accuracy is\n\
               estimated to be better than the critical value.\n\
return = -2 -> For at least one TI, the accuracy is estimated to be worse\n\
               than the critical values. The value of variable `flag` is\n\
               determined based on internal uncertainty estimations\n\
               performed by collier.");
static PyObject *
get_tis_accuracy_flag_py(PyObject *self, PyObject *args)
{

  int acc;
  get_tis_accuracy_flag_rcl(&acc);

  return Py_BuildValue("i", acc);
}
PyDoc_STRVAR(get_n_colour_configurations_doc,
"get_n_colour_configurations_rcl(npr)\n\n\
Returns the number of exisiting colour configurations\n\
for the process with process number `npr`.");
static PyObject *
get_n_colour_configurations_py(PyObject *self, PyObject *args)
{

  int npr,n;
  if (!PyArg_ParseTuple(args, "i", &npr))
    return NULL;

  get_n_colour_configurations_rcl(npr,&n);

  return Py_BuildValue("i", n);
  
}
PyDoc_STRVAR(get_colour_configuration_doc,
"get_colour_configuration_rcl(npr,n)\n\n\
Returns the colour configuration array associated to the colour \n\
configuration n of the process with process number `npr`.\n\
npr = integer -> process numer\n\
n   = integer -> n-th colour configuration");
static PyObject *
get_colour_configuration_py(PyObject *self, PyObject *args)
{

  int npr,n;
  if (!PyArg_ParseTuple(args, "ii", &npr, &n))
    return NULL;

  int legs;
  get_legs_rcl(npr,&legs);

  int col[legs];

  get_colour_configuration_rcl(npr,n,col);

  PyObject *col_py = PyList_New(legs);
  int i;
  for (i = 0; i < legs; i++) {
      PyList_SET_ITEM(col_py, i, int_to_pyobj(col[i]));
  }
  return col_py;
}
PyDoc_STRVAR(get_n_helicity_configurations_doc,
"get_n_helicity_configurations_rcl(npr)\n\n\
Returns the number of exisiting helicity configurations\n\
for the process with process number `npr`.");
static PyObject *
get_n_helicity_configurations_py(PyObject *self, PyObject *args)
{

  int npr,n;
  if (!PyArg_ParseTuple(args, "i", &npr))
    return NULL;

  get_n_helicity_configurations_rcl(npr,&n);

  return Py_BuildValue("i", n);
  
}
PyDoc_STRVAR(get_helicity_configuration_doc,
"get_helicity_configuration_rcl(npr,n)\n\n\
Returns the helicity configuration array associated to the helicity \n\
configuration n of the process with process number `npr`.\n\
npr = integer -> process numer\n\
n   = integer -> n-th helicity configuration");
static PyObject *
get_helicity_configuration_py(PyObject *self, PyObject *args)
{

  int npr,n;
  if (!PyArg_ParseTuple(args, "ii", &npr, &n))
    return NULL;

  int legs;
  get_legs_rcl(npr, &legs);

  int hel[legs];

  get_helicity_configuration_rcl(npr,n,hel);

  PyObject *hel_py = PyList_New(legs);
  int i;
  for (i = 0; i < legs; i++) {
      PyList_SET_ITEM(hel_py, i, int_to_pyobj(hel[i]));
  }
  return hel_py;
}


/************************
*  module wrapper_rcl  *
************************/

// set_alphas_masses_rcl -> recola1_interface_rcl
// use_gfermi_scheme_rcl -> recola1_interface_rcl
// use_alpha0_scheme_rcl -> recola1_interface_rcl
// use_alphaz_scheme_rcl -> recola1_interface_rcl

PyDoc_STRVAR(set_gs_power_doc,
"set_gs_power_rcl(npr, gsarray)\n\n\
Selects specific powers of g_s, as specified by the argument `gsarray`,\n\
for the process with identifier `npr`.\n\
npr     = integer -> process number\n\
gsarray = integer[:,2]\n\
        -> Integer array values can be 0 or 1,\n\
           with the first entry being the power of g_s,\n\
           and the second entry being the loop order\n\
           (0 for LO, 1 for NLO)");
static PyObject *
set_gs_power_py(PyObject *self, PyObject *args)
{

  int process_id;
  PyObject *gsarray_py;

  if (!PyArg_ParseTuple(args, "iO!", &process_id,
                        &PyList_Type, &gsarray_py))
      return NULL;

  const int numLines = PyList_Size(gsarray_py);
  int gsarray[numLines][2];

  PyObject *gs_py;

  int i;
  for (i=0; i<numLines; i++)
  {
    gs_py = PyList_GetItem(gsarray_py, i);
    PyObject *iter = PyObject_GetIter(gs_py);
    if (!iter)
    {
      // error not iterator
      return PyErr_Format(PyExc_Exception,
      "gsarray at position %d is not an iterator/list.", i);
    }
    int j = 0;
    while (1) {
      PyObject *next = PyIter_Next(iter);
      if (!next) {
        // nothing left in the iterator
        break;
      }
      if (!pyobj_pyint_check(next)) {
        // error, we were expecting a floating point value
      }
      gsarray[i][j] = pyobj_to_int(next);
      Py_DECREF(next);
      j = j + 1;
    }
    Py_DECREF(iter);
  }

  set_gs_power_rcl(process_id, gsarray, numLines);
  Py_INCREF(Py_None);
  return Py_None;
}
PyDoc_STRVAR(set_outgoing_momenta_doc,
"set_outgoing_momenta_rcl(npr,pIn)\n\n\
Random phase space generation given incoming momenta.\n\
npr    = integer -> process number\n\
pIn    = float[2][4] -> Incoming (beam) momenta\n\
return = float[l][4] -> random external momenta of the process with `l` legs");
static PyObject *
set_outgoing_momenta_py(PyObject *self, PyObject *args)
{
  int process_id;
  PyObject *momentaIn_py;

  if (!PyArg_ParseTuple(args, "iO!", &process_id, &PyList_Type, &momentaIn_py))
      return NULL;

  int legs;
  get_legs_rcl(process_id, &legs);

  int numLines;
  numLines = PyList_Size(momentaIn_py);
  double momentaIn[numLines][4];
  if(numLines != 2){
    return PyErr_Format(PyExc_Exception,
        "Incoming momenta pIn not of length 2. Momenta length given: %d",
        numLines);
  }


  PyObject *momentum_py;

  int i;
  for (i=0; i<numLines; i++)
  {
    momentum_py = PyList_GetItem(momentaIn_py, i);
    PyObject *iter = PyObject_GetIter(momentum_py);
    if (!iter)
    {
      // error, expected list/iterator
      return PyErr_Format(PyExc_Exception,
      "Given momenta %d is not of list/iterator type.", i);
    }
    int j = 0;
    while (1) {
      PyObject *next = PyIter_Next(iter);
      if (!next) {
        // nothing left in the iterator
        break;
      }
      if (!PyFloat_Check(next)) {
        // error, we were expecting a floating point value
      }
      momentaIn[i][j] = PyFloat_AsDouble(next);
      Py_DECREF(next);
      j = j + 1;
    }
    Py_DECREF(iter);
  }

  double momenta[legs][4];
  set_outgoing_momenta_rcl(process_id, momentaIn, momenta);

  PyObject *result = PyList_New(legs);
  for (i = 0; i < legs; i++) {
      PyObject *item = PyList_New(4);
      int j;
      for (j = 0; j < 4; j++)
          PyList_SET_ITEM(item, j, PyFloat_FromDouble(momenta[i][j]));
      PyList_SET_ITEM(result, i, item);
  }

  return result;
}
PyDoc_STRVAR(compute_process_doc,
"compute_process_rcl(npr,p,order)\n\n\
Computes the structure-dressed helicity amplitudes and the summed squared\n\
amplitude for the process with process number `npr`.\n\
npr   = integer -> process number\n\
p     = float[l][4] -> external momenta of the process with `l` legs\n\
order = string-> loop order, accepted values are 'LO' and 'NLO'\n\
return = (A_0^2, 2 Re[A_0 A_1], momenta_check)\n\n\
`momenta_check` tells whether the phase-space point is good\n\
(momenta_check=True) or bad (momenta_check=False) one.");
static PyObject *
compute_process_py(PyObject *self, PyObject *args)
{
  int process_id;
  const char* order;
  PyObject *momenta_py;

  if (!PyArg_ParseTuple(args, "iO!s", &process_id,
                        &PyList_Type, &momenta_py, &order))
      return (PyObject *) NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  double A2[2];
  int momcheck;
  compute_process_rcl(process_id, momenta, order, A2, &momcheck);

  return Py_BuildValue("ddO", A2[0], A2[1], momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(get_amplitude_doc,
"get_amplitude_rcl(npr,order,colour,hel,pow=None,gs=None)\n\n\
Extracts a specific contribution to the amplitude of the process with\n\
process number `npr`, according to the values of `pow`, `order`, `colour`\n\
and `hel`.\n\
npr    = integer -> process number\n\
order  = string\n\
       -> loop-order of the contribution:\n\
           string with accepted values:\n\
          'LO'     -> tree squared amplitude\n\
          'NLO'    -> loop squared amplitude\n\
          'NLO-D4' -> loop squared amplitude, bare loop contribution\n\
          'NLO-CT' -> loop squared amplitude, counterterms contribution\n\
          'NLO-R2' -> loop squared amplitude, rational terms contribution\n\
colour = integer[l]\n\
       -> colour structure which is a vector of type integer\n\
          and length l, where each position in the vector corresponds to\n\
          one of the l external particles of process `npr`\n\
          (ordered as in the process definition).\n\
          For colourless particles, incoming quarks and outgoing \n\
          anti-quarks, the corresponding entry in `colour` must be 0.\n\
          For all other particles (gluons, outgoing quarks and incoming\n\
          anti-quarks), the entries must be, without repetition, the\n\
          positions of the gluons, incoming quarks and outgoing\n\
          anti-quarks in the process definition.\n\
hel    = integer[l]\n\
       -> helicity of the contribution:\n\
          vector whose entries are the helicities of the particles:\n\
          left-handed  fermions/antifermions -> -1\n\
          right-handed fermions/antifermions -> +1\n\
          logitudinal vector bosons          ->  0\n\
          transverse vector bosons           -> -1, +1\n\
          scalar particles                   ->  0\n\
pow/gs = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of g_s only (power in e is determined\n\
           automatically from the underlying process)\n\
return  = (Ar, Ai)");
static PyObject *
get_amplitude_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,gspow;
  PyObject *pow_py, *gs_py;
  const char* order;
  PyObject *colour_py;
  PyObject *hel_py;
  int legs,npow,gs;
  dcomplex A;

  pow_py = Py_None;
  gs_py = Py_None;
  static char *kwlist[] = {"npr", "order", "colour", "helicity", "pow", "gs", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "isO!O!|O!O", kwlist,
                                   &process_id, &order,
                                   &PyList_Type, &colour_py,
                                   &PyList_Type, &hel_py,
                                   &PyList_Type, &pow_py, &gs_py))
      return (PyObject *) NULL;

  
  get_legs_rcl(process_id, &legs);
  int colour[legs];
  if (!build_c_integer_array(legs, colour_py, colour, "colour"))
  {
    return (PyObject *) NULL;
  }
  int hel[legs];
  if (!build_c_integer_array(legs, hel_py, hel, "hel"))
  {
    return (PyObject *) NULL;
  }
  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_amplitude_general_rcl(process_id, pow, order, colour, hel, &A);
  }
  else if (gs_py != Py_None)
  {
    if (!pyobj_pyint_check(gs_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "gs is not of type integer.");
      return (PyObject *) NULL;
    }
    gs = pyobj_to_int(gs_py);
    get_amplitude_r1_rcl(process_id, gs, order, colour, hel, &A);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor gs are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("dd", A.dr, A.di);
}
PyDoc_STRVAR(get_squared_amplitude_doc,
"get_squared_amplitude_rcl(npr,order,pow=None,als=None)\n\n\
Extracts the computed value of the summed squared amplitude with `pow` being\n\
order in the fundamental couplings at loop-order `order` for the process with\n\
process number `npr`.\n\
npr     = integer -> process number\n\
order   = string\n\
        -> loop-order of the contribution:\n\
            string with accepted values:\n\
           'LO'     -> tree squared amplitude\n\
           'NLO'    -> loop squared amplitude\n\
           'NLO-D4' -> loop squared amplitude, bare loop contribution\n\
           'NLO-CT' -> loop squared amplitude, counterterms contribution\n\
           'NLO-R2' -> loop squared amplitude, rational terms contribution\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
return = A2\n\
This routine can be called only after compute_process_rcl for process\n\
with process number `npr`");
static PyObject *
get_squared_amplitude_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,npow,as;
  PyObject *pow_py, *as_py;
  const char* order;
  double A2;

  pow_py = Py_None;
  as_py = Py_None;
  static char *kwlist[] = {"npr", "order", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "is|O!O", kwlist,
                                   &process_id, &order,
                                   &PyList_Type, &pow_py, &as_py))
      return (PyObject *)NULL;

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_squared_amplitude_general_rcl(process_id, pow, order, &A2);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_squared_amplitude_r1_rcl(process_id, as, order, &A2);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor als are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2);
}
PyDoc_STRVAR(get_polarized_squared_amplitude_doc,
"get_polarized_squared_amplitude_rcl(npr,order,hel,pow=None,als=None)\n\n\
Extracts a specific contribution to the polarized squared amplitude of the\n\
process with process number `npr`, according to the values of `pow`\n\
(or `als`), `order`, `colour` and `hel`.\n\
npr     = integer -> process number\n\
order   = string\n\
        -> loop-order of the contribution:\n\
            string with accepted values:\n\
           'LO'     -> tree squared amplitude\n\
           'NLO'    -> loop squared amplitude\n\
           'NLO-D4' -> loop squared amplitude, bare loop contribution\n\
           'NLO-CT' -> loop squared amplitude, counterterms contribution\n\
           'NLO-R2' -> loop squared amplitude, rational terms contribution\n\
hel     = integer[l]\n\
        -> helicity of the contribution:\n\
           vector whose entries are the helicities of the particles:\n\
           left-handed  fermions/antifermions -> -1\n\
           right-handed fermions/antifermions -> +1\n\
           logitudinal vector bosons          ->  0\n\
           transverse vector bosons           -> -1, +1\n\
           scalar particles                   ->  0\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
return = A2p");
static PyObject *
get_polarized_squared_amplitude_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,aspow;
  PyObject *pow_py, *as_py;
  const char* order;
  PyObject *hel_py;
  int legs,npow,as;
  double A2;
  Py_complex R;

  pow_py = Py_None;
  as_py = Py_None;
  static char *kwlist[] = {"npr", "order", "hel", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "isO!|O!O", kwlist,
                                   &process_id, &order,
                                   &PyList_Type, &hel_py,
                                   &PyList_Type, &pow_py, &as_py))
      return (PyObject *)NULL;

  get_legs_rcl(process_id, &legs);
  int hel[legs];
  if (!build_c_integer_array(legs, hel_py, hel, "hel"))
  {
    return (PyObject *) NULL;
  }

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_polarized_squared_amplitude_general_rcl(process_id, pow, order,
                                                hel, &A2);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_polarized_squared_amplitude_r1_rcl(process_id, as, order, hel, &A2);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor als are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2);
}
PyDoc_STRVAR(compute_colour_correlation_doc,
"compute_colour_correlation_rcl(npr,p,i1,i2,order)\n\n\
Computes the colour-correlated summed squared amplitude, between particle\n\
with leg number `i1` and particle with leg number `i2`, for the process with\n\
process number `npr`.\n\
npr = integer     -> process number\n\
p   = float[l][4] -> external momenta of the process with `l` legs\n\
i1  = integer     -> quark/anti-quark at leg i1\n\
i2  = integer     -> quark/anti-quark at leg i2\n\
return = (A2cc, momcheck)");
static PyObject *
compute_colour_correlation_py(PyObject *self, PyObject *args)
{
  int process_id, i1,i2;
  PyObject *momenta_py;
  const char* order;

  if (!PyArg_ParseTuple(args, "iO!iis", &process_id,
                        &PyList_Type, &momenta_py, &i1, &i2, &order))
      return (PyObject *)NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  double A2cc;
  int momcheck;
  compute_colour_correlation_rcl(process_id, momenta, i1, i2, order,
                                 &A2cc, &momcheck);

  return Py_BuildValue("dO", A2cc, momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(compute_colour_correlation_int_doc,
"compute_colour_correlation_int_rcl(npr,p,i1,i2)\n\n\
Computes the colour-correlated summed 1-loop interference amplitude\n\
between particle ith leg number `i1` and particle with leg number `i2`,\n\
for the process with process number `npr`.\n\
npr = integer     -> process number\n\
p   = float[l][4] -> external momenta of the process with `l` legs\n\
i1  = integer     -> quark/anti-quark at leg i1\n\
i2  = integer     -> quark/anti-quark at leg i2\n\
return = (A2ccint, momcheck)");
static PyObject *
compute_colour_correlation_int_py(PyObject *self, PyObject *args)
{
  int process_id, i1,i2;
  PyObject *momenta_py;

  if (!PyArg_ParseTuple(args, "iO!ii", &process_id,
                        &PyList_Type, &momenta_py, &i1, &i2))
      return (PyObject *)NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  double A2ccint;
  int momcheck;
  compute_colour_correlation_int_rcl(process_id, momenta, i1, i2,
                                     &A2ccint, &momcheck);

  return Py_BuildValue("dO", A2ccint, momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(compute_all_colour_correlations_doc,
"compute_all_colour_correlations_rcl(npr,p,order='LO')\n\n\
Computes the colour-correlated summed squared amplitude, between all\n\
possible pairs of coloured particles for the process with\n\
process number `npr`.\n\
npr    = integer     -> process number\n\
p      = float[l][4] -> external momenta of the process with `l` legs\n\
order  = string  -> either 'LO' (default) or 'NLO'\n\
return = momcheck");
static PyObject *
compute_all_colour_correlations_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id;
  PyObject *momenta_py, *order_py;
  order_py = Py_None;
  static char *kwlist[] = {"npr", "p", "order", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "iO!|O", kwlist,
                                   &process_id, &PyList_Type, &momenta_py,
                                   &order_py))
      return (PyObject *)NULL;

  const char *order;
  if (order_py != Py_None) {
    order = pyobj_to_string(order_py);
  } else {
    order = "LO";
  }
  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  int momcheck;
  compute_all_colour_correlations_rcl(process_id, momenta, order, &momcheck);

  return Py_BuildValue("O", momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(compute_all_colour_correlations_int_doc,
"compute_all_colour_correlations_int_rcl(npr,p)\n\n\
Computes the colour-correlated summed 1-loop interference amplitude,\n\
between all possible pairs of coloured particles for the process with\n\
process number `npr`.\n\
npr = integer     -> process number\n\
p   = float[l][4] -> external momenta of the process with `l` legs\n\
return = momcheck");
static PyObject *
compute_all_colour_correlations_int_py(PyObject *self, PyObject *args)
{
  int process_id;
  PyObject *momenta_py;

  if (!PyArg_ParseTuple(args, "iO!", &process_id,
                        &PyList_Type, &momenta_py))
      return (PyObject *)NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  int momcheck;
  compute_all_colour_correlations_int_rcl(process_id, momenta, &momcheck);

  return Py_BuildValue("O", momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(get_colour_correlation_doc,
"get_colour_correlation_rcl(npr,i1,i2,order='LO',pow=None,als=None)\n\n\
Extracts the computed value of the colour-correlated summed squared\n\
amplitude.\n\
npr     = integer -> process number\n\
i1      = integer -> quark/anti-quark at leg i1\n\
i2      = integer -> quark/anti-quark at leg i2\n\
order   = string  -> either 'LO' (default) or 'NLO'\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
return = A2cc\n\
This function can only be called after\n\
compute_colour_correlation_rcl or rescale_colour_correlation_rcl\n\
(for process with process number `npr` for particles `i1` and `i2`)\n\
or after compute_all_colour_correlations_rcl for process or\n\
rescale_all_colour_correlations_rcl for process with process number `npr`.");
static PyObject *
get_colour_correlation_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,as,i1,i2,npow;
  PyObject *order_py, *pow_py, *as_py;
  double A2cc;

  pow_py = Py_None;
  as_py = Py_None;
  order_py = Py_None;
  static char *kwlist[] = {"npr", "i1", "i2", "order", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "iii|OO!O", kwlist,
                                   &process_id, &i1, &i2,
                                   &order_py, &PyList_Type, &pow_py, &as_py))
      return (PyObject *) NULL;

  const char *order;
  if (order_py != Py_None) {
    order = pyobj_to_string(order_py);
  } else {
    order = "LO";
  }

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_colour_correlation_general_rcl(process_id, pow, i1, i2, order, &A2cc);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_colour_correlation_r1_rcl(process_id, as, i1, i2, order, &A2cc);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor gs are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2cc);
}
PyDoc_STRVAR(get_colour_correlation_int_doc,
"get_colour_correlation_int_rcl(npr,i1,i2,pow=None,als=None)\n\n\
Extracts the computed value of the colour-correlated summed 1-loop \n\
interference amplitude.\n\
npr = integer -> process number\n\
i1  = integer -> quark/anti-quark at leg i1\n\
i2  = integer -> quark/anti-quark at leg i2\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
return = A2ccint\n\
This function can only be called after\n\
compute_colour_correlation_int_rcl or rescale_colour_correlation_int_rcl\n\
(for process with process number `npr` for particles `i1` and `i2`)\n\
or after compute_all_colour_correlation_ints_rcl for process or\n\
rescale_all_colour_correlation_ints_rcl for process with process number `npr`.");
static PyObject *
get_colour_correlation_int_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,as,i1,i2,npow;
  PyObject *pow_py, *as_py;
  double A2ccint;

  pow_py = Py_None;
  as_py = Py_None;
  static char *kwlist[] = {"npr", "i1", "i2", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "iii|O!O", kwlist,
                                   &process_id, &i1, &i2,
                                   &PyList_Type, &pow_py, &as_py))
      return (PyObject *) NULL;

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_colour_correlation_int_general_rcl(process_id, pow, i1, i2, &A2ccint);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_colour_correlation_int_r1_rcl(process_id, as, i1, i2, &A2ccint);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor gs are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2ccint);
}
PyDoc_STRVAR(compute_spin_colour_correlation_doc,
"compute_spin_colour_correlation_rcl(npr,p,i1,i2,v)\n\n\
Computes the spin-colour-correlated summed squared amplitude, between\n\
particle with leg number `i1` and particle with leg number `i2`, for the\n\
process with process number `npr`.\n\
The spin-correlation is achieved by a user-defined polarization vector `v`\n\
for the particle with leg number `i1`, which substitutes the usual one.\n\
npr = integer     -> process number\n\
p   = float[l][4] -> external momenta of the process with `l` legs\n\
i1  = integer     -> quark/anti-quark at leg i1\n\
i2  = integer     -> quark/anti-quark at leg i2\n\
v   = complex[4]  -> polarization vector\n\
return  = (A2scc, momcheck)");
static PyObject *
compute_spin_colour_correlation_py(PyObject *self, PyObject *args)
{
  int process_id, i1,i2;
  PyObject *momenta_py;
  PyObject *polvec_py;

  if (!PyArg_ParseTuple(args, "iO!iiO!", &process_id,
                        &PyList_Type, &momenta_py, &i1,&i2,
                        &PyList_Type, &polvec_py))
      return (PyObject *) NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  dcomplex v[4];
  if (!build_c_polvec(polvec_py, v))
  {
    return (PyObject *) NULL;
  }

  double A2scc;
  int momcheck;
  compute_spin_colour_correlation_rcl(process_id, momenta, i1, i2, v,
                                      &A2scc, &momcheck);

  return Py_BuildValue("dO", A2scc, momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(get_spin_colour_correlation_doc,
"get_spin_colour_correlation_rcl(npr,i1,i2,pow=None,als=None)\n\n\
Extracts the computed value of the spin-colour-correlated summed squared\n\
amplitude.\n\
npr = integer -> process number\n\
i1  = integer -> quark/anti-quark at leg i1\n\
i2  = integer -> quark/anti-quark at leg i2\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
This function can only be called after\n\
compute_spin_colour_correlation_rcl or rescale_spin_colour_correlation_rcl\n\
(for process with process number `npr` for particles `i1` and `i2`)\n\
or after compute_all_spin_colour_correlations_rcl for process or\n\
rescale_all_spin_colour_correlations_rcl for process with process\n\
number `npr`.");
static PyObject *
get_spin_colour_correlation_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,as,i1,i2,npow;
  PyObject *pow_py, *as_py;
  double A2scc;

  pow_py = Py_None;
  as_py = Py_None;
  static char *kwlist[] = {"npr", "i1", "i2", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "iii|O!O", kwlist,
                                   &process_id, &i1, &i2,
                                   &PyList_Type, &pow_py, &as_py))

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_spin_colour_correlation_general_rcl(process_id, pow, i1, i2, &A2scc);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_spin_colour_correlation_r1_rcl(process_id, as, i1, i2, &A2scc);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor als are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2scc);
}
PyDoc_STRVAR(compute_spin_correlation_doc,
"compute_spin_correlation_rcl(npr,p,j,v)\n\n\
Computes the spin-correlated summed squared amplitude, where the \n\
polarization vector of particle `j`, necessarily being a gluon or photon,\n\
is is substituted for the user-defined polarization vector `v`.\n\
npr = integer     -> process number\n\
p   = float[l][4] -> external momenta of the process with `l` legs\n\
j   = integer     -> photon/gluon leg\n\
v   = complex[4]  -> photon/gluon polarization vector\n\
return  = (A2sc, momcheck)");
static PyObject *
compute_spin_correlation_py(PyObject *self, PyObject *args)
{
  int process_id, j;
  PyObject *momenta_py;
  PyObject *polvec_py;

  if (!PyArg_ParseTuple(args, "iO!iO!", &process_id,
                        &PyList_Type, &momenta_py, &j,
                        &PyList_Type, &polvec_py))
    return (PyObject *) NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  if (!build_c_momenta(process_id, momenta_py, momenta))
  {
    return (PyObject *) NULL;
  }

  dcomplex v[4];
  if (!build_c_polvec(polvec_py, v))
  {
    return (PyObject *) NULL;
  }

  double A2sc;
  int momcheck;
  compute_spin_correlation_rcl(process_id, momenta, j, v, &A2sc, &momcheck);

  return Py_BuildValue("dO", A2sc, momcheck ? Py_True: Py_False);
}
PyDoc_STRVAR(get_spin_correlation_doc,
"get_spin_correlation_rcl(npr,pow=None,als=None)\n\n\
Extracts the computed value of the spin-correlated summed squared\n\
amplitude.\n\
npr     = integer -> process number\n\
pow/als = list/integer\n\
        -> array with values representing the powers in fundamental\n\
           couplings in the order as dictated by the model file, e.g.\n\
           `[2, 3] = g_s^2 e^3`, or, if supported by the model,\n\
           the power in terms of als=g_s^2 only (power in e is determined\n\
           automatically from the underlying process)\n\
This routine can be called only after\n\
compute_spin_correlation_rcl or rescale_spin_correlation_rcl\n\
for process with process number `npr`.");
static PyObject *
get_spin_correlation_py(PyObject *self, PyObject *args, PyObject *keywds)
{
  int process_id,as,npow;
  PyObject *pow_py, *as_py;
  double A2sc;

  pow_py = Py_None;
  as_py = Py_None;
  static char *kwlist[] = {"npr", "pow", "als", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, keywds, "i|O!O", kwlist,
                                   &process_id,
                                   &PyList_Type, &pow_py, &as_py))

  if (pow_py != Py_None) {
    // reconstruct pow_py
    get_n_orders_rcl(&npow);
    int pow[npow];
    if (!build_c_integer_array(npow, pow_py, pow, "pow"))
    {
      return (PyObject *) NULL;
    }
    get_spin_correlation_general_rcl(process_id,pow,&A2sc);
  }
  else if (as_py != Py_None)
  {
    if (!pyobj_pyint_check(as_py)) {
      // error, we are expecting an integer value
      PyErr_SetString(PyExc_Exception, "als is not of type integer.");
      return (PyObject *) NULL;
    }
    as = pyobj_to_int(as_py);
    get_spin_correlation_r1_rcl(process_id,as,&A2sc);
  }
  else
  {
    PyErr_SetString(PyExc_Exception, "Neither pow nor als are != None.");
    return (PyObject *) NULL;
  }
  return Py_BuildValue("d", A2sc);
}
PyDoc_STRVAR(get_momenta_doc,
"get_momenta_rcl(npr)\n\n\
Extracts the stored momenta of the process with process number `npr`.\n\
Note that the internal momenta can differ from the ones given by the user.\n\
This function can only be called after:\n\
 - compute_process_rcl\n\
 - compute_colour_correlation_rcl\n\
 - compute_all_colour_correlations_rcl\n\
 - compute_spin_colour_correlation_rcl\n\
 - compute_spin_correlation_rcl\n\
npr    = integer -> process number\n\
return = p[l][4] -> Internal momenta used in the calculation.");
static PyObject *
get_momenta_py(PyObject *self, PyObject *args)
{
  int process_id;
  if (!PyArg_ParseTuple(args, "i", &process_id))
      return (PyObject *) NULL;

  int legs;
  get_legs_rcl(process_id, &legs);
  double momenta[legs][4];
  get_momenta_rcl(process_id, momenta);

  PyObject *result = PyList_New(legs);
  int i;
  for (i = 0; i < legs; i++) {
      PyObject *item = PyList_New(4);
      int j;
      for (j = 0; j < 4; j++)
          PyList_SET_ITEM(item, j, PyFloat_FromDouble(momenta[i][j]));
      PyList_SET_ITEM(result, i, item);
  }
  return result;
}
PyDoc_STRVAR(get_recola_version_doc,
"get_recola_version_rcl()\n\n\
Returns the version number of the Recola2 library.");
static PyObject *
get_recola_version_py(PyObject *self, PyObject *args)
{
  char recolaversion[10];
  get_recola_version_rcl(&recolaversion);
  return Py_BuildValue("s", recolaversion);
}
PyDoc_STRVAR(get_modelname_doc,
"get_modelname_rcl()\n\n\
Returns the name of model linked to the Recola2 library.");
static PyObject *
get_modelname_py(PyObject *self, PyObject *args)
{
  char modelname[100];
  get_modelname_rcl(&modelname);
  return Py_BuildValue("s", modelname);
}
PyDoc_STRVAR(get_driver_timestamp_doc,
"get_driver_timestamp_rcl()\n\n\
Returns the precise generation date of model library.");
static PyObject *
get_driver_timestamp_py(PyObject *self, PyObject *args)
{
  char driverts[200];
  get_driver_timestamp_rcl(&driverts);
  return Py_BuildValue("s", driverts);
}
PyDoc_STRVAR(get_renoscheme_doc,
"get_renoscheme_rcl(rname)\n\n\
Returns the active renormalization scheme for the parameter `rname`.\n\
rname = string");
static PyObject *
get_renoscheme_py(PyObject *self, PyObject *args)
{
  const char* rname;

  if (!PyArg_ParseTuple(args, "s", &rname))
      return (PyObject *)NULL;

  char rscheme[100];
  get_renoscheme_rcl(rname, &rscheme);
  return Py_BuildValue("s", rscheme);
}

/*****************************************
*  module extended_higgs_interface_rcl  *
*****************************************/

PyDoc_STRVAR(set_Z2_thdm_yukawa_type_doc,
"set_Z2_thdm_yukawa_type_rcl(ytype)\n\n\
Sets the yukawa type in the softly-broken Z2 symmetric THDM to `ytype`.\n\
ytype = 1 -> Type-I   (h1u=h1d=h1l=0)\n\
ytype = 2 -> Type-II  (h1u=h2d=h2l=0)\n\
ytype = 3 -> Type-X   (h1u=h1d=h2l=0)\n\
ytype = 4 -> Type-Y   (h1u=h2d=h1l=0)");
static PyObject *
set_Z2_thdm_yukawa_type_py(PyObject *self, PyObject *args)
{

  int ytype;
  if (!PyArg_ParseTuple(args, "i", &ytype))
      return NULL;

  set_Z2_thdm_yukawa_type_rcl(ytype);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_hl_doc,
"set_pole_mass_hl_rcl(m,g)\n\n\
Sets the pole mass and width of the light Higgs boson Hl.\n\
m = pole mass \n\
w = pole width");
static PyObject *
set_pole_mass_hl_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_hl_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_hh_doc,
"set_pole_mass_hh_rcl(m,g)\n\n\
Sets the pole mass and width of the heavy Higgs boson hh.\n\
m = pole mass \n\
w = pole width");
static PyObject *
set_pole_mass_hh_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_hh_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_hl_hh_doc,
"set_pole_mass_hl_hh_rcl(ml,wl,mh,wh)\n\n\
Sets the pole masses, widths of the light and heavy Higgs bosons\n\
to `ml`, `wl` and `mh`, `wh`, respectively.\n\
ml = pole mass of light Higgs boson\n\
wl = pole width of light Higgs boson\n\
mh = pole mass of light Higgs boson\n\
wh = pole width of light Higgs boson");
static PyObject *
set_pole_mass_hl_hh_py(PyObject *self, PyObject *args)
{

  double ml, gl, mh, gh;
  if (!PyArg_ParseTuple(args, "dddd", &ml, &gl, &mh, &gh))
      return NULL;

  set_pole_mass_hl_hh_rcl(ml, gl, mh, gh);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_ha_doc,
"set_pole_mass_ha_rcl(m,g)\n\n\
Sets the pole mass and width of the pseudo-scalar Higgs boson ha.\n\
m = pole mass\n\
w = pole width");
static PyObject *
set_pole_mass_ha_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_ha_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_pole_mass_hc_doc,
"set_pole_mass_hc_rcl(m,g)\n\n\
Sets the pole mass and width of the charged Higgs boson hc.\n\
m = pole mass\n\
w = pole width");
static PyObject *
set_pole_mass_hc_py(PyObject *self, PyObject *args)
{

  double m, g;
  if (!PyArg_ParseTuple(args, "dd", &m, &g))
      return NULL;

  set_pole_mass_hc_rcl(m, g);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_tb_cab_doc,
"set_tb_cab_rcl(tb,cab)\n\n\
Sets the for tan(b) and cos(a-b) to `tb` and `cab`, respectively.\n\
tb = tan(b) ratio of the vevs vev2/vev1\n\
cab = cos(a-b) mixing angle");
static PyObject *
set_tb_cab_py(PyObject *self, PyObject *args)
{

  double tb, cab;
  if (!PyArg_ParseTuple(args, "dd", &tb, &cab))
      return NULL;

  set_tb_cab_rcl(tb, cab);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_mixing_alpha_msbar_scheme_doc,
"use_mixing_alpha_msbar_scheme_rcl(s)\n\n\
Sets the renormalization scheme for the mixing angle alpha or a related\n\
parameter to the msbar scheme.\n\
s = 'FJ' -> msbar, tadpoles in FJ tadpole scheme\n\
s = 'GB' -> msbar, tadpoles defined `generic basis` scheme\n\
s = 'SMC'-> msbar, tadpoles defined in `shifted minimum condition scheme`\n\
s = 'l3' -> alpha traded for l3 and l3 renormalized msbar\n\
s = 'l345' -> alpha traded for l345:=l3+l4+l5 and l345 renormalized msbar");
static PyObject *
use_mixing_alpha_msbar_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_mixing_alpha_msbar_scheme_rcl(s);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_mixing_alpha_onshell_scheme_doc,
"use_mixing_alpha_onshell_scheme_rcl(s)\n\n\
Sets the renormalization scheme for the mixing angle alpha\n\
to an on-shell scheme defined by the neutral Higgs mixing.\n\
s = 'ps' -> p^* scheme\n\
s = 'os' -> on-shell scheme defined via mixing field constants");
static PyObject *
use_mixing_alpha_onshell_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_mixing_alpha_onshell_scheme_rcl(s);
  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_mixing_beta_onshell_scheme_doc,
"use_mixing_beta_onshell1_scheme_rcl(s)\n\n\
Sets the renormalization scheme for the mixing angle beta\n\
to the on-shell scheme defined via either the\n\
  (1) pseudo scalar Higgs - pseudo goldstone boson mixing\n\
  (2) charged Higgs - charged goldstone boson mixing\n\
s = 'ps1' -> p^* scheme for (1)\n\
s = 'ps2' -> p^* scheme for (2)\n\
s = 'os1' -> on-shell scheme via mixing field constants for (1)\n\
s = 'os2' -> on-shell scheme via mixing field constants for (2)");
static PyObject *
use_mixing_beta_onshell_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_mixing_beta_onshell_scheme_rcl(s);
  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_mixing_beta_msbar_scheme_doc,
"use_mixing_beta_msbar_scheme_rcl(s)\n\n\
Sets the renormalization scheme for the mixing angle beta to the msbar scheme\n\
s = 'FJ' -> msbar, tadpoles in FJ tadpole scheme\n\
s = 'TS' -> tadpole subtracted msbar scheme, equivalent to 'GB' and 'SMB'");
static PyObject *
use_mixing_beta_msbar_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_mixing_beta_msbar_scheme_rcl(s);
  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_msb_doc,
"set_msb_rcl(msb)\n\n\
Sets the value for the soft-Z2 breaking scale to `msb`.");
static PyObject *
set_msb_py(PyObject *self, PyObject *args)
{

  double msb;
  if (!PyArg_ParseTuple(args, "d", &msb))
      return NULL;

  set_msb_rcl(msb);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_msb_msbar_scheme_doc,
"use_msb_msbar_scheme_rcl(s)\n\n\
Sets the renormalization scheme for soft-Z2 breaking\n\
scale or a related paraemter to the msbar scheme.\n\
s = 'MSB' -> MSB renormalized msbar\n\
s = 'm12' -> m12 renormalized msbar");
static PyObject *
use_msb_msbar_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_msb_msbar_scheme_rcl(s);
  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_sa_doc,
"set_sa_rcl(sa)\n\n\
Sets the value for the mixing angle sin(a) to `sa`.");
static PyObject *
set_sa_py(PyObject *self, PyObject *args)
{

  double sa;
  if (!PyArg_ParseTuple(args, "d", &sa))
      return NULL;

  set_sa_rcl(sa);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(set_tb_doc,
"set_tb_rcl(tb)\n\n\
Sets the value for tan(b) to `tb`.");
static PyObject *
set_tb_py(PyObject *self, PyObject *args)
{

  double tb;
  if (!PyArg_ParseTuple(args, "d", &tb))
      return NULL;

  set_tb_rcl(tb);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(use_tb_msbar_scheme_doc,
"use_tb_msbar_scheme_rcl(s)\n\n\
Sets the renormalization scheme for tb to the msbar scheme.\n\
s = 'FJ' -> msbar, tadpoles in FJ tadpole scheme\n\
s = 'TS' -> tadpole subtracted msbar scheme, NOT equivalent to 'GB' or 'SMB'");
static PyObject *
use_tb_msbar_scheme_py(PyObject *self, PyObject *args)
{
  const char* s;
  if (!PyArg_ParseTuple(args, "s", &s))
      return NULL;

  use_tb_msbar_scheme_rcl(s);
  Py_INCREF(Py_None);
  return Py_None; 
}

/***************************
*  module statistics_rcl  *
***************************/
PyDoc_STRVAR(print_generation_statistics_doc,
"print_generation_statistics_rcl()\n\n\
Prints the CPU time spent for the process generation.");
static PyObject *
print_generation_statistics_py(PyObject *self, PyObject *args)
{
  print_generation_statistics_rcl();
  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(print_TI_statistics_doc,
"print_TI_statistics_rcl(npr,npoints)\n\n\
Prints the CPU time spent for the computation of tensor integrals.\n\
npr     = integer -> process number\n\
npoints = integer -> number of ps (normalization)");
static PyObject *
print_TI_statistics_py(PyObject *self, PyObject *args)
{

  int npr, npoints;
  if (!PyArg_ParseTuple(args, "ii", &npr, &npoints))
      return NULL;

  print_TI_statistics_rcl(npr, npoints);

  Py_INCREF(Py_None);
  return Py_None; 
}
PyDoc_STRVAR(print_TC_statistics_doc,
"print_TC_statistics_rcl(npr,npoints)\n\n\
Prints the CPU time spent for the computation of tensor coefficients.\n\
npr     = integer -> process number\n\
npoints = integer -> number of ps (normalization)");
static PyObject *
print_TC_statistics_py(PyObject *self, PyObject *args)
{

  int npr, npoints;
  if (!PyArg_ParseTuple(args, "ii", &npr, &npoints))
      return NULL;

  print_TC_statistics_rcl(npr, npoints);

  Py_INCREF(Py_None);
  return Py_None; 
}


/******************
*  module reset  *
******************/

static PyObject *
reset_recola_py(PyObject *self, PyObject *args)
{
  reset_recola_rcl();
  Py_INCREF(Py_None);
  return Py_None;
}


/*********************
*  pyrecola module  *
*********************/

static PyMethodDef pyrecola_methods[] = {
  // module input_rcl
  {"set_qcd_rescaling_rcl", set_qcd_rescaling_py, METH_VARARGS, set_qcd_rescaling_doc},
  {"set_fermionloop_optimization_rcl", set_fermionloop_optimization_py, METH_VARARGS,set_fermionloop_optimization_doc},
  {"set_helicity_optimization_rcl", set_helicity_optimization_py, METH_VARARGS,set_helicity_optimization_doc},
  {"set_colour_optimization_rcl", set_colour_optimization_py, METH_VARARGS,set_colour_optimization_doc},
  {"set_compute_A12_rcl", set_compute_A12_py, METH_VARARGS,set_compute_A12_doc},
  {"set_masscut_rcl", set_masscut_py, METH_VARARGS, set_masscut_doc},
  {"set_light_particle_rcl", set_light_particle_py, METH_VARARGS, set_light_particle_doc},
  {"unset_light_particle_rcl", unset_light_particle_py, METH_VARARGS, unset_light_particle_doc},
  {"use_dim_reg_soft_rcl", use_dim_reg_soft_py, METH_VARARGS, use_dim_reg_soft_doc},
  {"use_mass_reg_soft_rcl", use_mass_reg_soft_py, METH_VARARGS, use_mass_reg_soft_doc},
  {"use_recola_base_rcl", use_recola_base_py, METH_VARARGS, use_recola_base_doc},
  {"set_mass_reg_soft_rcl", set_mass_reg_soft_py, METH_VARARGS, set_mass_reg_soft_doc},
  {"set_delta_uv_rcl", set_delta_uv_py, METH_VARARGS, set_delta_uv_doc},
  {"get_delta_uv_rcl", get_delta_uv_py, METH_VARARGS, get_delta_uv_doc},
  {"set_delta_ir_rcl", set_delta_ir_py, METH_VARARGS, set_delta_ir_doc},
  {"get_delta_ir_rcl", get_delta_ir_py, METH_VARARGS, get_delta_ir_doc},
  {"set_mu_uv_rcl", set_mu_uv_py, METH_VARARGS, set_mu_uv_doc},
  {"get_mu_uv_rcl", get_mu_uv_py, METH_VARARGS, get_mu_uv_doc},
  {"set_mu_ir_rcl", set_mu_ir_py, METH_VARARGS, set_mu_ir_doc},
  {"get_mu_ir_rcl", get_mu_ir_py, METH_VARARGS, get_mu_ir_doc},
  {"set_mu_ms_rcl", set_mu_ms_py, METH_VARARGS, set_mu_ms_doc},
  {"get_mu_ms_rcl", get_mu_ms_py, METH_VARARGS, get_mu_ms_doc},
  {"get_renormalization_scale_rcl", get_renormalization_scale_py, METH_VARARGS, get_renormalization_scale_doc},
  {"get_flavour_scheme_rcl", get_flavour_scheme_py, METH_VARARGS, get_flavour_scheme_doc},
  {"set_complex_mass_scheme_rcl", set_complex_mass_scheme_py, METH_VARARGS, set_complex_mass_scheme_doc},
  {"set_on_shell_scheme_rcl", set_on_shell_scheme_py, METH_VARARGS, set_on_shell_scheme_doc},
  {"set_momenta_correction_rcl", set_momenta_correction_py, METH_VARARGS, set_momenta_correction_doc},
  {"set_print_level_amplitude_rcl", set_print_level_amplitude_py, METH_VARARGS, set_print_level_amplitude_doc},
  {"set_print_level_squared_amplitude_rcl", set_print_level_squared_amplitude_py, METH_VARARGS, set_print_level_squared_amplitude_doc},
  {"set_print_level_correlations_rcl", set_print_level_correlations_py, METH_VARARGS, set_print_level_correlations_doc},
  {"set_print_level_parameters_rcl", set_print_level_parameters_py, METH_VARARGS, set_print_level_parameters_doc},
  {"set_draw_level_branches_rcl", set_draw_level_branches_py, METH_VARARGS, set_draw_level_branches_doc},
  {"set_print_level_RAM_rcl", set_print_level_RAM_py, METH_VARARGS, set_print_level_RAM_doc},
  {"scale_coupling3_rcl", scale_coupling3_py, METH_VARARGS, scale_coupling3_doc},
  {"scale_coupling4_rcl", scale_coupling4_py, METH_VARARGS, scale_coupling4_doc},
  {"switchoff_coupling3_rcl", switchoff_coupling3_py, METH_VARARGS, switchoff_coupling3_doc},
  {"switchoff_coupling4_rcl", switchoff_coupling4_py, METH_VARARGS, switchoff_coupling4_doc},
  {"switchon_resonant_selfenergies_rcl", switchon_resonant_selfenergies_py, METH_VARARGS, switchon_resonant_selfenergies_doc},
  {"switchoff_resonant_selfenergies_rcl", switchoff_resonant_selfenergies_py, METH_VARARGS, switchoff_resonant_selfenergies_doc},
  {"set_dynamic_settings_rcl", set_dynamic_settings_py, METH_VARARGS, set_dynamic_settings_doc},
  {"set_ifail_rcl", set_ifail_py, METH_VARARGS, set_ifail_doc},
  {"get_ifail_rcl", get_ifail_py, METH_VARARGS, get_ifail_doc},
  {"set_iwarn_rcl", set_iwarn_py, METH_VARARGS, set_iwarn_doc},
  {"get_iwarn_rcl", get_iwarn_py, METH_VARARGS, get_iwarn_doc},
  {"set_collier_output_dir_rcl", set_collier_output_dir_py, METH_VARARGS,set_collier_output_dir_doc},
  {"set_output_file_rcl", set_output_file_py, METH_VARARGS,set_output_file_doc},
  {"set_log_mem_rcl", set_log_mem_py, METH_VARARGS, set_log_mem_doc},
  {"set_init_collier_rcl", set_init_collier_py, METH_VARARGS, set_init_collier_doc},
  {"set_parameter_rcl", set_parameter_py, METH_VARARGS, set_parameter_doc},
  {"get_parameter_rcl", get_parameter_py, METH_VARARGS, get_parameter_doc},
  {"set_renoscheme_rcl", set_renoscheme_py, METH_VARARGS, set_renoscheme_doc},
  {"set_resonant_particle_rcl", set_resonant_particle_py, METH_VARARGS, set_resonant_particle_doc},
  {"set_quarkline_rcl", set_quarkline_py, METH_VARARGS, set_quarkline_doc},
  {"reset_vertices_rcl", reset_vertices_py, METH_VARARGS, reset_vertices_doc},
  {"reset_couplings_rcl", reset_couplings_py, METH_VARARGS, reset_couplings_doc},
  {"reset_ctcouplings_rcl", reset_ctcouplings_py, METH_VARARGS, reset_ctcouplings_doc},
  {"print_collier_statistics_rcl", print_collier_statistics_py, METH_VARARGS, print_collier_statistics_doc},
  // additional stuff, only used in combination with rept1l
  {"set_compute_selfenergy_rcl", set_compute_selfenergy_py, METH_VARARGS},
  {"set_compute_selfenergy_offshell_rcl", set_compute_selfenergy_offshell_py, METH_VARARGS},
  {"set_compute_tadpole_rcl", set_compute_tadpole_py, METH_VARARGS},
  {"set_lp_rcl", set_lp_py, METH_VARARGS},
  {"set_form_rcl", set_form_py, METH_VARARGS},
  {"set_all_couplings_active_rcl", set_all_couplings_active_py, METH_VARARGS},
  {"set_all_tree_couplings_active_rcl", set_all_tree_couplings_active_py, METH_VARARGS},
  {"set_all_loop_couplings_active_rcl", set_all_loop_couplings_active_py, METH_VARARGS},
  {"set_all_ct_couplings_active_rcl", set_all_ct_couplings_active_py, METH_VARARGS},
  {"set_all_r2_couplings_active_rcl", set_all_r2_couplings_active_py, METH_VARARGS},
  {"set_complex_mass_form_rcl", set_complex_mass_form_py, METH_VARARGS},
  {"set_vertex_functions_rcl", set_vertex_functions_py, METH_VARARGS},
  {"disable_particle_ordering_rcl", disable_particle_ordering_py, METH_VARARGS},
  {"set_masscut_form_rcl", set_masscut_form_py, METH_VARARGS},
  {"set_cutparticle_rcl", set_cutparticle_py, METH_VARARGS},
  {"unset_cutparticle_rcl", unset_cutparticle_py, METH_VARARGS},
  {"set_print_recola_logo_rcl", set_print_recola_logo_py, METH_VARARGS},
  // module recola1_interface_rcl
  {"set_pole_mass_w_rcl", set_pole_mass_w_py, METH_VARARGS, set_pole_mass_w_doc},
  {"set_onshell_mass_w_rcl", set_onshell_mass_w_py, METH_VARARGS, set_onshell_mass_w_doc},
  {"set_pole_mass_z_rcl", set_pole_mass_z_py, METH_VARARGS, set_pole_mass_z_doc},
  {"set_onshell_mass_z_rcl", set_onshell_mass_z_py, METH_VARARGS, set_onshell_mass_z_doc},
  {"set_pole_mass_top_rcl", set_pole_mass_top_py, METH_VARARGS, set_pole_mass_top_doc},
  {"set_pole_mass_bottom_rcl", set_pole_mass_bottom_py, METH_VARARGS, set_pole_mass_bottom_doc},
  {"set_pole_mass_charm_rcl", set_pole_mass_charm_py, METH_VARARGS, set_pole_mass_charm_doc},
  {"set_pole_mass_strange_rcl", set_pole_mass_strange_py, METH_VARARGS, set_pole_mass_strange_doc},
  {"set_pole_mass_up_rcl", set_pole_mass_up_py, METH_VARARGS, set_pole_mass_up_doc},
  {"set_pole_mass_down_rcl", set_pole_mass_down_py, METH_VARARGS, set_pole_mass_down_doc},
  {"set_pole_mass_h_rcl", set_pole_mass_h_py, METH_VARARGS, set_pole_mass_h_doc},
  {"set_pole_mass_tau_rcl", set_pole_mass_tau_py, METH_VARARGS, set_pole_mass_tau_doc},
  {"set_pole_mass_muon_rcl", set_pole_mass_muon_py, METH_VARARGS, set_pole_mass_muon_doc},
  {"set_pole_mass_electron_rcl", set_pole_mass_electron_py, METH_VARARGS, set_pole_mass_electron_doc},
  {"set_light_top_rcl", set_light_top_py, METH_VARARGS, set_light_top_doc},
  {"unset_light_top_rcl", unset_light_top_py, METH_VARARGS, unset_light_top_doc},
  {"set_light_bottom_rcl", set_light_bottom_py, METH_VARARGS, set_light_bottom_doc},
  {"unset_light_bottom_rcl", unset_light_bottom_py, METH_VARARGS, unset_light_bottom_doc},
  {"set_light_charm_rcl", set_light_charm_py, METH_VARARGS, set_light_charm_doc},
  {"unset_light_charm_rcl", unset_light_charm_py, METH_VARARGS, unset_light_charm_doc},
  {"set_light_strange_rcl", set_light_strange_py, METH_VARARGS, set_light_strange_doc},
  {"unset_light_strange_rcl", unset_light_strange_py, METH_VARARGS, unset_light_strange_doc},
  {"set_light_up_rcl", set_light_up_py, METH_VARARGS, set_light_up_doc},
  {"unset_light_up_rcl", unset_light_up_py, METH_VARARGS, unset_light_up_doc},
  {"set_light_down_rcl", set_light_down_py, METH_VARARGS, set_light_down_doc},
  {"unset_light_down_rcl", unset_light_down_py, METH_VARARGS, unset_light_down_doc},
  {"set_light_tau_rcl", set_light_tau_py, METH_VARARGS, set_light_tau_doc},
  {"unset_light_tau_rcl", unset_light_tau_py, METH_VARARGS, unset_light_tau_doc},
  {"set_light_muon_rcl", set_light_muon_py, METH_VARARGS, set_light_muon_doc},
  {"unset_light_muon_rcl", unset_light_muon_py, METH_VARARGS, unset_light_muon_doc},
  {"set_light_electron_rcl", set_light_electron_py, METH_VARARGS, set_light_electron_doc},
  {"unset_light_electron_rcl", unset_light_electron_py, METH_VARARGS, unset_light_electron_doc},
  {"set_light_fermions_rcl", set_light_fermions_py, METH_VARARGS, set_light_fermions_doc},
  {"use_gfermi_scheme_rcl", (PyCFunction)use_gfermi_scheme_py, METH_VARARGS|METH_KEYWORDS, use_gfermi_scheme_doc},
  {"use_alpha0_scheme_rcl", (PyCFunction)use_alpha0_scheme_py, METH_VARARGS|METH_KEYWORDS, use_alpha0_scheme_doc},
  {"use_alphaZ_scheme_rcl", (PyCFunction)use_alphaZ_scheme_py, METH_VARARGS|METH_KEYWORDS, use_alphaZ_scheme_doc},
  {"get_alpha_rcl", get_alpha_py, METH_VARARGS, get_alpha_doc},
  {"set_alphas_rcl", set_alphas_py, METH_VARARGS, set_alphas_doc},
  {"get_alphas_rcl", get_alphas_py, METH_VARARGS, get_alphas_doc},
  {"set_alphas_masses_rcl", (PyCFunction)set_alphas_masses_py, METH_VARARGS|METH_KEYWORDS, set_alphas_masses_doc},
  {"compute_running_alphas_rcl", compute_running_alphas_py, METH_VARARGS, compute_running_alphas_doc},
  {"select_all_gs_powers_BornAmpl_rcl", select_all_gs_powers_BornAmpl_py, METH_VARARGS, select_all_gs_powers_BornAmpl_doc},
  {"unselect_all_gs_powers_BornAmpl_rcl", unselect_all_gs_powers_BornAmpl_py, METH_VARARGS, unselect_all_gs_powers_BornAmpl_doc},
  {"select_gs_power_BornAmpl_rcl", select_gs_power_BornAmpl_py, METH_VARARGS, select_gs_power_BornAmpl_doc},
  {"unselect_gs_power_BornAmpl_rcl", unselect_gs_power_BornAmpl_py, METH_VARARGS, unselect_gs_power_BornAmpl_doc},
  {"select_all_gs_powers_LoopAmpl_rcl", select_all_gs_powers_LoopAmpl_py, METH_VARARGS, select_all_gs_powers_LoopAmpl_doc},
  {"unselect_all_gs_powers_LoopAmpl_rcl", unselect_all_gs_powers_LoopAmpl_py, METH_VARARGS, unselect_all_gs_powers_LoopAmpl_doc},
  {"select_gs_power_LoopAmpl_rcl", select_gs_power_LoopAmpl_py, METH_VARARGS, select_gs_power_LoopAmpl_doc},
  {"unselect_gs_power_LoopAmpl_rcl", unselect_gs_power_LoopAmpl_py, METH_VARARGS, unselect_gs_power_LoopAmpl_doc},
  // module process_definition_rcl
  {"define_process_rcl", define_process_py, METH_VARARGS, define_process_doc},
  {"select_power_BornAmpl_rcl", select_power_BornAmpl_py, METH_VARARGS, select_power_BornAmpl_doc},
  {"unselect_power_BornAmpl_rcl", unselect_power_BornAmpl_py, METH_VARARGS, unselect_power_BornAmpl_doc},
  {"select_all_powers_BornAmpl_rcl", select_all_powers_BornAmpl_py, METH_VARARGS, select_all_powers_BornAmpl_doc},
  {"unselect_all_powers_BornAmpl_rcl", unselect_all_powers_BornAmpl_py, METH_VARARGS, unselect_all_powers_BornAmpl_doc},
  {"select_power_LoopAmpl_rcl", select_power_LoopAmpl_py, METH_VARARGS, select_power_LoopAmpl_doc},
  {"unselect_power_LoopAmpl_rcl", unselect_power_LoopAmpl_py, METH_VARARGS, unselect_power_LoopAmpl_doc},
  {"select_all_powers_LoopAmpl_rcl", select_all_powers_LoopAmpl_py, METH_VARARGS, select_all_powers_LoopAmpl_doc},
  {"unselect_all_powers_LoopAmpl_rcl", unselect_all_powers_LoopAmpl_py, METH_VARARGS, unselect_all_powers_LoopAmpl_doc},
  {"split_collier_cache_rcl", split_collier_cache_py, METH_VARARGS, split_collier_cache_doc},
  // module process_generation_rcl
  {"generate_processes_rcl", generate_processes_py, METH_VARARGS, generate_processes_doc},
  // module process_computation_rcl
  {"set_resonant_squared_momentum_rcl", set_resonant_squared_momentum_py, METH_VARARGS, set_resonant_squared_momentum_doc},
  {"rescale_process_rcl", rescale_process_py, METH_VARARGS, rescale_process_doc},
  {"rescale_colour_correlation_rcl", (PyCFunction)rescale_colour_correlation_py, METH_VARARGS|METH_KEYWORDS, rescale_colour_correlation_doc},
  {"rescale_colour_correlation_int_rcl", rescale_colour_correlation_int_py, METH_VARARGS, rescale_colour_correlation_int_doc},
  {"rescale_all_colour_correlations_rcl", rescale_all_colour_correlations_py, METH_VARARGS, rescale_all_colour_correlations_doc},
  {"rescale_all_colour_correlations_int_rcl", rescale_all_colour_correlations_int_py, METH_VARARGS, rescale_all_colour_correlations_int_doc},
  {"rescale_spin_colour_correlation_rcl", rescale_spin_colour_correlation_py, METH_VARARGS, rescale_spin_colour_correlation_doc},
  {"rescale_spin_correlation_rcl", rescale_spin_correlation_py, METH_VARARGS, rescale_spin_correlation_doc},
  {"set_tis_required_accuracy_rcl", set_tis_required_accuracy_py, METH_VARARGS, set_tis_required_accuracy_doc},
  {"get_tis_required_accuracy_rcl", get_tis_required_accuracy_py, METH_VARARGS, get_tis_required_accuracy_doc},
  {"set_tis_critical_accuracy_rcl", set_tis_critical_accuracy_py, METH_VARARGS, set_tis_critical_accuracy_doc},
  {"get_tis_critical_accuracy_rcl", get_tis_critical_accuracy_py, METH_VARARGS, get_tis_critical_accuracy_doc},
  {"get_tis_accuracy_flag_rcl", get_tis_accuracy_flag_py, METH_VARARGS, get_tis_accuracy_flag_doc},
  {"get_n_colour_configurations_rcl", get_n_colour_configurations_py, METH_VARARGS, get_n_colour_configurations_doc},
  {"get_colour_configuration_rcl", get_colour_configuration_py, METH_VARARGS, get_colour_configuration_doc},
  {"get_n_helicity_configurations_rcl", get_n_helicity_configurations_py, METH_VARARGS, get_n_helicity_configurations_doc},
  {"get_helicity_configuration_rcl", get_helicity_configuration_py, METH_VARARGS, get_helicity_configuration_doc},
  // module wrapper_rcl
  {"set_gs_power_rcl", set_gs_power_py, METH_VARARGS, set_gs_power_doc},
  {"set_outgoing_momenta_rcl", set_outgoing_momenta_py, METH_VARARGS, set_outgoing_momenta_doc},
  {"compute_process_rcl", compute_process_py, METH_VARARGS, compute_process_doc},
  {"get_amplitude_rcl", (PyCFunction)get_amplitude_py, METH_VARARGS|METH_KEYWORDS, get_amplitude_doc},
  {"get_squared_amplitude_rcl", (PyCFunction)get_squared_amplitude_py, METH_VARARGS|METH_KEYWORDS, get_squared_amplitude_doc},
  {"get_polarized_squared_amplitude_rcl", (PyCFunction)get_polarized_squared_amplitude_py, METH_VARARGS|METH_KEYWORDS, get_polarized_squared_amplitude_doc},
  {"compute_colour_correlation_rcl", compute_colour_correlation_py, METH_VARARGS, compute_colour_correlation_doc},
  {"compute_colour_correlation_int_rcl", compute_colour_correlation_int_py, METH_VARARGS, compute_colour_correlation_int_doc},
  {"compute_all_colour_correlations_rcl", (PyCFunction)compute_all_colour_correlations_py, METH_VARARGS|METH_KEYWORDS, compute_all_colour_correlations_doc},
  {"compute_all_colour_correlations_int_rcl", compute_all_colour_correlations_int_py, METH_VARARGS, compute_all_colour_correlations_int_doc},
  {"get_colour_correlation_rcl", (PyCFunction)get_colour_correlation_py, METH_VARARGS|METH_KEYWORDS, get_colour_correlation_doc},
  {"get_colour_correlation_int_rcl", (PyCFunction)get_colour_correlation_int_py, METH_VARARGS|METH_KEYWORDS, get_colour_correlation_int_doc},
  {"compute_spin_colour_correlation_rcl", compute_spin_colour_correlation_py, METH_VARARGS, compute_spin_colour_correlation_doc},
  {"get_spin_colour_correlation_rcl", (PyCFunction)get_spin_colour_correlation_py, METH_VARARGS|METH_KEYWORDS, get_spin_colour_correlation_doc},
  {"compute_spin_correlation_rcl", compute_spin_correlation_py, METH_VARARGS},
  {"get_spin_correlation_rcl", (PyCFunction)get_spin_correlation_py, METH_VARARGS|METH_KEYWORDS, get_spin_correlation_doc},
  {"get_momenta_rcl", get_momenta_py, METH_VARARGS, get_momenta_doc},
  {"get_recola_version_rcl", get_recola_version_py, METH_VARARGS, get_recola_version_doc},
  {"get_modelname_rcl", get_modelname_py, METH_VARARGS, get_modelname_doc},
  {"get_driver_timestamp_rcl", get_driver_timestamp_py, METH_VARARGS, get_driver_timestamp_doc},
  {"get_renoscheme_rcl", get_renoscheme_py, METH_VARARGS, get_renoscheme_doc},
  // thdm interface
  {"set_Z2_thdm_yukawa_type_rcl", set_Z2_thdm_yukawa_type_py, METH_VARARGS, set_Z2_thdm_yukawa_type_doc},
  {"set_pole_mass_hl_rcl", set_pole_mass_hl_py, METH_VARARGS, set_pole_mass_hl_doc},
  {"set_pole_mass_hh_rcl", set_pole_mass_hh_py, METH_VARARGS, set_pole_mass_hh_doc},
  {"set_pole_mass_hl_hh_rcl", set_pole_mass_hl_hh_py, METH_VARARGS, set_pole_mass_hl_hh_doc},
  {"set_pole_mass_ha_rcl", set_pole_mass_ha_py, METH_VARARGS, set_pole_mass_ha_doc},
  {"set_pole_mass_hc_rcl", set_pole_mass_hc_py, METH_VARARGS, set_pole_mass_hc_doc},
  {"set_tb_cab_rcl", set_tb_cab_py, METH_VARARGS, set_tb_cab_doc},
  {"use_mixing_alpha_msbar_scheme_rcl", use_mixing_alpha_msbar_scheme_py, METH_VARARGS, use_mixing_alpha_msbar_scheme_doc},
  {"use_mixing_alpha_onshell_scheme_rcl", use_mixing_alpha_onshell_scheme_py, METH_VARARGS, use_mixing_alpha_onshell_scheme_doc},
  {"use_mixing_beta_onshell_scheme_rcl", use_mixing_beta_onshell_scheme_py, METH_VARARGS, use_mixing_beta_onshell_scheme_doc},
  {"use_mixing_beta_msbar_scheme_rcl", use_mixing_beta_msbar_scheme_py, METH_VARARGS, use_mixing_beta_msbar_scheme_doc},
  {"set_msb_rcl", set_msb_py, METH_VARARGS, set_msb_doc},
  {"use_msb_msbar_scheme_rcl", use_msb_msbar_scheme_py, METH_VARARGS, use_msb_msbar_scheme_doc},
  // hs interface
  {"set_sa_rcl", set_sa_py, METH_VARARGS, set_sa_doc},
  {"set_tb_rcl", set_tb_py, METH_VARARGS, set_tb_doc},
  {"use_tb_msbar_scheme_rcl", use_tb_msbar_scheme_py, METH_VARARGS, use_tb_msbar_scheme_doc},
  // statistics_rcl
  {"print_generation_statistics_rcl", print_generation_statistics_py, METH_VARARGS, print_generation_statistics_doc},
  {"print_TI_statistics_rcl", print_TI_statistics_py, METH_VARARGS, print_TI_statistics_doc},
  {"print_TC_statistics_rcl", print_TC_statistics_py, METH_VARARGS, print_TC_statistics_doc},
  // reset_rcl
  {"reset_recola_rcl", reset_recola_py, METH_VARARGS},
  {NULL,NULL}
};

#if PYTHON3
  #define MOD_ERROR_VAL NULL
  #define MOD_SUCCESS_VAL(val) val
  #define MOD_INIT(name) PyMODINIT_FUNC PyInit_##name(void)
  #define MOD_DEF(ob, name, doc, methods) \
          static struct PyModuleDef moduledef = { \
            PyModuleDef_HEAD_INIT, name, doc, -1, methods, }; \
          ob = PyModule_Create(&moduledef);
#else
  #define MOD_ERROR_VAL
  #define MOD_SUCCESS_VAL(val)
  #define MOD_INIT(name) void init##name(void)
  #define MOD_DEF(ob, name, doc, methods) \
          ob = Py_InitModule3(name, methods, doc);
#endif

MOD_INIT(pyrecola)
{
  PyObject *m;

  MOD_DEF(m, "pyrecola", pyrecola_doc, pyrecola_methods)
  if(m == NULL) return MOD_ERROR_VAL;

  return MOD_SUCCESS_VAL(m);
}
#ifdef __cplusplus
}  // extern "C"
#endif 
